@@ -14,6 +14,8 @@
  * do a proper job of it), some of the as now unused stuff might come
  * in handy.
  *				- kev 4/7/1998
+ *
+ * $Header: /usr/build/vile/vile/RCS/api.c,v 1.52 2015/09/07 00:53:13 tom Exp $
  */
 
 #include "estruct.h"
@@ -51,16 +53,8 @@ linsert_chars(char *s, int len)
 	if (s[len - 1] == '\n') {
 	    if (len > 1)
 		nlsuppress = TRUE;
-	    if (!mdnewline) {
-		make_local_b_val(curbp, MDNEWLINE);
-		set_b_val(curbp, MDNEWLINE, TRUE);
-	    }
-	} else {
-	    if (mdnewline) {
-		make_local_b_val(curbp, MDNEWLINE);
-		set_b_val(curbp, MDNEWLINE, FALSE);
-	    }
 	}
+	set_local_b_val(curbp, MDNEWLINE, !mdnewline);
     }
 
     if (s[len - 1] == '\n') {
@@ -68,8 +62,7 @@ linsert_chars(char *s, int len)
 	    && lforw(DOT.l) == buf_head(curbp)
 	    && DOT.o == llength(DOT.l)) {
 	    nlsuppress = TRUE;
-	    make_local_b_val(curbp, MDNEWLINE);
-	    set_b_val(curbp, MDNEWLINE, TRUE);
+	    set_local_b_val(curbp, MDNEWLINE, TRUE);
 	}
 	if (!nlsuppress) {
 
@@ -101,9 +94,9 @@ linsert_chars(char *s, int len)
 	     * can't be a newline.
 	     */
 	    for (i = 1; i < len && s[i] != '\n'; i++) ;
-	    linsert(i, *s);
+	    lins_bytes(i, *s);
 	    if (i > 1) {
-		memcpy(DOT.l->l_text + DOT.o - i + 1, s + 1, i - 1);
+		memcpy(lvalue(DOT.l) + DOT.o - i + 1, s + 1, (size_t) (i - 1));
 	    }
 	    len -= i;
 	    s += i;
@@ -112,8 +105,7 @@ linsert_chars(char *s, int len)
 
     if (nlatend) {
 	/* Advance DOT to where it's supposed to be */
-	DOT.l = lforw(DOT.l);
-	DOT.o = 0;
+	dot_next_bol();
     }
 
     return nlcount;
@@ -128,33 +120,33 @@ lreplace(char *s, int len)
     char *t;
     WINDOW *wp;
 
-    copy_for_undo(lp);
+    CopyForUndo(lp);
 
-    DOT.o = 0;
+    DOT.o = b_left_margin(curbp);
 
     if (len > (int) lp->l_size) {
-	int nlen;
+	size_t nlen;
 	char *ntext;
 
 #define roundlenup(n) ((n+NBLOCK-1) & ~(NBLOCK-1))
 
-	nlen = roundlenup(len);
+	nlen = (size_t) roundlenup(len);
 	ntext = castalloc(char, nlen);
 	if (ntext == 0)
 	    return FALSE;
-	if (lp->l_text)
+	if (lvalue(lp))
 	    ltextfree(lp, curbp);
-	lp->l_text = ntext;
+	lvalue(lp) = ntext;
 	lp->l_size = nlen;
 #if OPT_LINE_ATTRS
 	FreeAndNull(lp->l_attrs);
 #endif
     }
 
-    lp->l_used = len;
+    llength(lp) = len;
 
     /* FIXME: Handle embedded newlines */
-    for (i = len - 1, t = lp->l_text; i >= 0; i--)
+    for (i = len - 1, t = lvalue(lp); i >= 0; i--)
 	t[i] = s[i];
 
     /* We'd normally need a call to chg_buff here, but I don't want
@@ -202,12 +194,11 @@ api_setup_fake_win(VileBuf * vbp, int do_delete)
 	if (vbp->ndel != 0 && do_delete) {
 	    int status;
 	    /* Do lazy delete; FALSE means don't put text in kill buffer */
-	    status = ldelete(vbp->ndel, FALSE);
+	    status = ldel_bytes(vbp->ndel, FALSE);
 	    vbp->ndel = 0;
 	    if (status == FALSE
 		|| (lforw(DOT.l) == buf_head(curbp) && DOT.o >= llength(DOT.l))) {
-		make_local_b_val(curbp, MDNEWLINE);
-		set_b_val(curbp, MDNEWLINE, FALSE);
+		set_local_b_val(curbp, MDNEWLINE, FALSE);
 	    }
 	}
     }
@@ -300,7 +291,7 @@ api_gotoline(VileBuf * vbp, int lno)
 	    break;
     }
 
-    DOT.o = 0;
+    DOT.o = b_left_margin(curbp);
 
     if (lp != buf_head(bp) && lno == lp->l_number) {
 	DOT.l = lp;
@@ -311,7 +302,7 @@ api_gotoline(VileBuf * vbp, int lno)
     }
 
 #else
-    return gotoline(TRUE, lno);
+    return vl_gotoline(lno);
 #endif
 }
 
@@ -325,7 +316,7 @@ api_aline(VileBuf * vbp, int lno, char *line, int len)
 	linsert_chars(line, len);
 	lnewline();
     } else {			/* append to the end */
-	gotoline(FALSE, 0);
+	gotoeob(FALSE, 0);
 	gotoeol(FALSE, 0);
 	lnewline();
 	linsert_chars(line, len);
@@ -349,12 +340,11 @@ api_dotinsert(VileBuf * vbp, char *text, int len)
     linsert_chars(text, len);
     if (vbp->ndel) {
 	int status;
-	status = ldelete(vbp->ndel, FALSE);
+	status = ldel_bytes(vbp->ndel, FALSE);
 	vbp->ndel = 0;
 	if (status == FALSE
 	    || (lforw(DOT.l) == buf_head(curbp) && DOT.o >= llength(DOT.l))) {
-	    make_local_b_val(curbp, MDNEWLINE);
-	    set_b_val(curbp, MDNEWLINE, FALSE);
+	    set_local_b_val(curbp, MDNEWLINE, FALSE);
 	}
     }
     vbp->changed = 1;
@@ -371,7 +361,7 @@ api_dline(VileBuf * vbp, int lno)
     if (lno > 0 && lno <= vl_line_count(vbp->bp)) {
 	api_gotoline(vbp, lno);
 	gotobol(TRUE, TRUE);
-	status = ldelete(llength(DOT.l) + 1, FALSE);
+	status = ldel_bytes((B_COUNT) (llength(DOT.l) + 1), FALSE);
     } else
 	status = FALSE;
 
@@ -381,16 +371,17 @@ api_dline(VileBuf * vbp, int lno)
 int
 api_gline(VileBuf * vbp, int lno, char **linep, int *lenp)
 {
+    static char empty[1];
     int status = TRUE;
 
     api_setup_fake_win(vbp, TRUE);
 
     if (lno > 0 && lno <= vl_line_count(vbp->bp)) {
 	api_gotoline(vbp, lno);
-	*linep = DOT.l->l_text;
+	*linep = lvalue(DOT.l);
 	*lenp = llength(DOT.l);
 	if (*lenp == 0) {
-	    *linep = "";	/* Make sure we pass back a zero length,
+	    *linep = empty;	/* Make sure we pass back a zero length,
 				   null terminated value when the length
 				   is zero.  Otherwise perl gets confused.
 				   (It thinks it should calculate the length
@@ -423,18 +414,18 @@ api_dotgline(VileBuf * vbp, char **linep, B_COUNT * lenp, int *neednewline)
     }
 
     if ((DOT.l == vbp->region.r_end.l
-	 && (vbp->regionshape == FULLLINE
-	     || (vbp->regionshape == EXACT
+	 && (vbp->regionshape == rgn_FULLLINE
+	     || (vbp->regionshape == rgn_EXACT
 		 && DOT.o >= vbp->region.r_end.o)))) {
 	return FALSE;
     }
 
-    *linep = DOT.l->l_text + DOT.o;
+    *linep = lvalue(DOT.l) + DOT.o;
 
     if (llength(DOT.l) >= DOT.o) {
 	*lenp = llength(DOT.l) - DOT.o;
 
-	if (vbp->regionshape == EXACT && DOT.l == vbp->region.r_end.l) {
+	if (vbp->regionshape == rgn_EXACT && DOT.l == vbp->region.r_end.l) {
 	    B_COUNT next = *lenp + vbp->region.r_end.o;
 	    if (next >= (B_COUNT) llength(DOT.l)) {
 		*lenp = next - llength(DOT.l);
@@ -456,7 +447,7 @@ api_dotgline(VileBuf * vbp, char **linep, B_COUNT * lenp, int *neednewline)
     }
 
     if (vbp->inplace_edit) {
-	if (vbp->regionshape == EXACT && DOT.l == vbp->region.r_end.l) {
+	if (vbp->regionshape == rgn_EXACT && DOT.l == vbp->region.r_end.l) {
 	    vbp->ndel = *lenp;
 	    *neednewline = 0;
 	} else {
@@ -464,12 +455,11 @@ api_dotgline(VileBuf * vbp, char **linep, B_COUNT * lenp, int *neednewline)
 	    *neednewline = 1;
 	}
     } else {
-	if (vbp->regionshape == EXACT && DOT.l == vbp->region.r_end.l) {
+	if (vbp->regionshape == rgn_EXACT && DOT.l == vbp->region.r_end.l) {
 	    DOT.o += *lenp;
 	    *neednewline = 0;
 	} else {
-	    DOT.l = lforw(DOT.l);
-	    DOT.o = 0;
+	    dot_next_bol();
 	    *neednewline = 1;
 	}
     }
@@ -486,9 +476,9 @@ api_sline(VileBuf * vbp, int lno, char *line, int len)
 
     if (lno > 0 && lno <= vl_line_count(vbp->bp)) {
 	api_gotoline(vbp, lno);
-	if (DOT.l->l_text != line
+	if (lvalue(DOT.l) != line
 	    && (llength(DOT.l) != len
-		|| memcmp(line, DOT.l->l_text, len) != 0)) {
+		|| memcmp(line, lvalue(DOT.l), (size_t) len) != 0)) {
 	    lreplace(line, len);
 	    vbp->changed = 1;
 	}
@@ -535,7 +525,7 @@ api_delregion(VileBuf * vbp)
     MK = vbp->region.r_end;
     regionshape = vbp->regionshape;
 
-    if (vbp->regionshape == FULLLINE) {
+    if (vbp->regionshape == rgn_FULLLINE) {
 	MK.l = lback(MK.l);
     }
 
@@ -570,11 +560,11 @@ api_motion(VileBuf * vbp, char *mstr)
 
     mp = mstr + strlen(mstr);
 
-    mapungetc(esc_c | NOREMAP);
+    mapungetc((int) ((unsigned char) esc_c | NOREMAP));
     /* Should we allow remapping?  Seems to me like it introduces too
        many variables. */
     while (mp-- > mstr) {
-	mapungetc(*mp | NOREMAP);
+	mapungetc((int) ((unsigned char) *mp | NOREMAP));
     }
 
     while (mapped_ungotc_avail()) {
@@ -681,7 +671,9 @@ api_swscreen(VileBuf * oldsp, VileBuf * newsp)
      * buried in the macro "for_each_visible_window".
      *          - kev 4/20/1998
      */
-    TRACE((T_CALLED "api_swscreen(oldsp=%p, newsp=%p)\n", oldsp, newsp));
+    TRACE((T_CALLED "api_swscreen(oldsp=%p, newsp=%p)\n",
+	   (void *) oldsp,
+	   (void *) newsp));
 
     curwp = curwp_visible ? curwp_visible : curwp;
     curbp = curwp->w_bufp;
@@ -699,6 +691,8 @@ api_swscreen(VileBuf * oldsp, VileBuf * newsp)
 void
 api_update(void)
 {
+    TRACE((T_CALLED "api_update\n"));
+
     propagate_dot();
 
     curwp = curwp_visible ? curwp_visible : curwp;
@@ -707,6 +701,8 @@ api_update(void)
     update(TRUE);
 
     curwp_visible = curwp;
+
+    returnVoid();
 }
 
 /*
@@ -809,7 +805,7 @@ api_free_private(void *vsp)
 
     if (vbp) {
 	vbp->bp->b_api_private = 0;
-#if OPT_PERL
+#if OPT_PERL && !NO_LEAKS
 	perl_free_handle(vbp->perl_handle);
 #endif
 	free(vbp);
@@ -830,7 +826,7 @@ api_bp2vbp(BUFFER *bp)
 	if (vbp != 0) {
 	    bp->b_api_private = vbp;
 	    vbp->bp = bp;
-	    vbp->regionshape = FULLLINE;
+	    vbp->regionshape = rgn_FULLLINE;
 	    vbp->region.r_orig.l =
 		vbp->region.r_end.l = buf_head(bp);
 	    vbp->region.r_orig.o =
