@@ -2,24 +2,27 @@
  * Prototypes for functions in the mode-tables (must be declared before the
  * point at which proto.h is included).
  *
- * $Header: /usr/build/vile/vile/RCS/chgdfunc.h,v 1.17 2005/01/15 16:33:33 cmorgan Exp $
+ * $Header: /usr/build/vile/vile/RCS/chgdfunc.h,v 1.23 2008/11/08 13:40:12 tom Exp $
  */
 extern int chgd_autobuf  (CHGD_ARGS);
 extern int chgd_buffer   (CHGD_ARGS);
 extern int chgd_charset  (CHGD_ARGS);
 extern int chgd_disabled (CHGD_ARGS);
+extern int chgd_dos_mode (CHGD_ARGS);
 extern int chgd_fences   (CHGD_ARGS);
 extern int chgd_hilite   (CHGD_ARGS);
 extern int chgd_major    (CHGD_ARGS);
-extern int chgd_major_w  (CHGD_ARGS);
 extern int chgd_mouse    (CHGD_ARGS);
+extern int chgd_percent  (CHGD_ARGS);
 extern int chgd_rs       (CHGD_ARGS);
 extern int chgd_status   (CHGD_ARGS);
 extern int chgd_undoable (CHGD_ARGS);
 extern int chgd_uniqbuf  (CHGD_ARGS);
+extern int chgd_win_mode (CHGD_ARGS);
 extern int chgd_window   (CHGD_ARGS);
 extern int chgd_working  (CHGD_ARGS);
 extern int chgd_xterm    (CHGD_ARGS);
+extern int chgd_xtermkeys(CHGD_ARGS);
 
 #if OPT_COLOR
 extern int chgd_color    (CHGD_ARGS);
@@ -29,13 +32,22 @@ extern int chgd_color    (CHGD_ARGS);
 extern int chgd_scheme   (CHGD_ARGS);
 #endif
 
+#if OPT_CURTOKENS   
+extern int chgd_curtokens(CHGD_ARGS);
+#endif
+
+#if OPT_FINDPATH
+extern int chgd_find_cfg(CHGD_ARGS);
+#endif
+
 #if OPT_MAJORMODE
 extern int chgd_mm_order (CHGD_ARGS);
 extern int chgd_filter   (CHGD_ARGS);
 #endif
 
-#if OPT_FINDPATH
-extern int chgd_find_cfg(CHGD_ARGS);
+#if OPT_MULTIBYTE
+extern int chgd_byteorder(CHGD_ARGS);
+extern int chgd_fileencode(CHGD_ARGS);
 #endif
 
 #if OPT_TITLE
