@@ -46,7 +46,7 @@
  * vile will choose some appropriate fallback (such as underlining) if
  * italics are not available.
  *
- * $Header: /usr/build/vile/vile/filters/RCS/manfilt.c,v 1.36 2004/10/27 19:18:30 tom Exp $
+ * $Header: /usr/build/vile/vile/filters/RCS/manfilt.c,v 1.68 2014/10/17 00:35:42 tom Exp $
  *
  */
 
@@ -57,6 +57,23 @@
 #if OPT_LOCALE
 #include	<locale.h>
 #include	<ctype.h>
+
+#undef sys_isdigit
+#undef sys_isprint
+
+#ifdef HAVE_WCTYPE
+#include	<wctype.h>
+#define sys_isdigit(n)  (iswdigit((wint_t)(n)) || (((wint_t)(n) < 256 && isdigit((wint_t)(n)))))
+#define sys_isprint(n)  (iswprint((wint_t)(n)) || (((wint_t)(n) < 256 && isprint((wint_t)(n)))))
+#else
+#define sys_isdigit(n)  isdigit(n)
+#define sys_isprint(n)  isprint(n)
+#endif
+
+#else
+#define sys_isdigit(n)  isdigit(n)
+#define sys_isprint(n)  isprint(n)
+
 #endif
 
 #if !defined(HAVE_STDLIB_H)
@@ -106,7 +123,7 @@ typedef struct CharCell {
     int c_attrs;		/* base cell only: ATR_NORMAL, etc. */
     char c_ident;		/* CS_NORMAL/CS_ALTERNATE */
     char c_level;		/* 0=base, 1=up halfline, -1=down halfline */
-    char c_value;		/* the actual value */
+    unsigned c_value;		/* the actual value */
 } CHARCELL;
 
 typedef struct LineData {
@@ -123,11 +140,9 @@ static void flush_line(void);
 static LINEDATA *all_lines;
 static LINEDATA *cur_line;
 static long total_lines;
+static int opt_8bit = 0;
 static int (*my_getc) (FILE *);
-
-#if OPT_LOCALE
-static int utf8_locale = 0;
-#endif
+static int (*my_putc) (int);
 
 static void
 failed(const char *s)
@@ -139,7 +154,13 @@ failed(const char *s)
 static int
 ansi_getc(FILE *fp)
 {
-    return fgetc(fp);
+    return vl_getc(fp);
+}
+
+static int
+ansi_putc(int ch)
+{
+    return vl_putc(ch, stdout);
 }
 
 #if OPT_LOCALE
@@ -154,7 +175,7 @@ utf8_getc(FILE *fp)
     int utf_char = 0;
 
     do {
-	int c = fgetc(fp);
+	int c = vl_getc(fp);
 	/* Combine UTF-8 into Unicode */
 	if (c < 0x80) {
 	    /* We received an ASCII character */
@@ -211,26 +232,99 @@ utf8_getc(FILE *fp)
 	    }
 	}
     } while (utf_count > 0);
-    if (utf_char > 255) {
-	switch (utf_char) {
-	case 0x2018:
-	    utf_char = '`';
-	    break;
-	case 0x2019:
-	    utf_char = '\'';
-	    break;
-	case 0x2010:
-	case 0x2212:
-	    utf_char = '-';
-	    break;
-	default:
-	    utf_char = '?';
-	    break;
+
+    if (opt_8bit) {
+	if (utf_char > 255) {
+	    switch (utf_char) {
+	    case 0x2018:
+		utf_char = '`';
+		break;
+	    case 0x2019:
+		utf_char = '\'';
+		break;
+	    case 0x2010:
+	    case 0x2212:
+		utf_char = '-';
+		break;
+	    default:
+		utf_char = '?';
+		break;
+	    }
 	}
     }
     return utf_char;
 }
-#endif
+
+static int
+utf8_putc(int source)
+{
+    /* FIXME - this is cut/paste from vl_conv_to_utf8() */
+#define CH(n) CharOf((source) >> ((n) * 8))
+    int rc = 0;
+    unsigned char target[10];
+
+    if (source <= 0x0000007f)
+	rc = 1;
+    else if (source <= 0x000007ff)
+	rc = 2;
+    else if (source <= 0x0000ffff)
+	rc = 3;
+    else if (source <= 0x001fffff)
+	rc = 4;
+    else if (source <= 0x03ffffff)
+	rc = 5;
+    else			/* (source <= 0x7fffffff) */
+	rc = 6;
+
+    switch (rc) {
+    case 1:
+	target[0] = CH(0);
+	break;
+
+    case 2:
+	target[1] = CharOf(0x80 | (CH(0) & 0x3f));
+	target[0] = CharOf(0xc0 | (CH(0) >> 6) | ((CH(1) & 0x07) << 2));
+	break;
+
+    case 3:
+	target[2] = CharOf(0x80 | (CH(0) & 0x3f));
+	target[1] = CharOf(0x80 | (CH(0) >> 6) | ((CH(1) & 0x0f) << 2));
+	target[0] = CharOf(0xe0 | ((int) (CH(1) & 0xf0) >> 4));
+	break;
+
+    case 4:
+	target[3] = CharOf(0x80 | (CH(0) & 0x3f));
+	target[2] = CharOf(0x80 | (CH(0) >> 6) | ((CH(1) & 0x0f) << 2));
+	target[1] = CharOf(0x80 |
+			   ((int) (CH(1) & 0xf0) >> 4) |
+			   ((int) (CH(2) & 0x03) << 4));
+	target[0] = CharOf(0xf0 | ((int) (CH(2) & 0x1f) >> 2));
+	break;
+
+    case 5:
+	target[4] = CharOf(0x80 | (CH(0) & 0x3f));
+	target[3] = CharOf(0x80 | (CH(0) >> 6) | ((CH(1) & 0x0f) << 2));
+	target[2] = CharOf(0x80 |
+			   ((int) (CH(1) & 0xf0) >> 4) |
+			   ((int) (CH(2) & 0x03) << 4));
+	target[1] = CharOf(0x80 | (CH(2) >> 2));
+	target[0] = CharOf(0xf8 | (CH(3) & 0x03));
+	break;
+
+    case 6:
+	target[5] = CharOf(0x80 | (CH(0) & 0x3f));
+	target[4] = CharOf(0x80 | (CH(0) >> 6) | ((CH(1) & 0x0f) << 2));
+	target[3] = CharOf(0x80 | (CH(1) >> 4) | ((CH(2) & 0x03) << 4));
+	target[2] = CharOf(0x80 | (CH(2) >> 2));
+	target[1] = CharOf(0x80 | (CH(3) & 0x3f));
+	target[0] = CharOf(0xfc | ((int) (CH(3) & 0x40) >> 6));
+	break;
+    }
+    target[rc] = 0;
+    return vl_fputs((char *) target, stdout);
+#undef CH
+}
+#endif /* OPT_LOCALE */
 
 /*
  * Allocate a CHARCELL struct
@@ -279,18 +373,20 @@ extend_line(void)
 	} else {
 	    c = typereallocn(CHARCELL, c, want);
 	}
-	if (c == 0)
+	if (c == 0) {
 	    failed("extend_line");
-	while (have < want) {
-	    c[have].link = 0;
-	    c[have].c_attrs = ATR_NORMAL;
-	    c[have].c_value = SPACE;
-	    c[have].c_level = 0;
-	    c[have].c_ident = CS_NORMAL;
-	    have++;
+	} else {
+	    while (have < want) {
+		c[have].link = 0;
+		c[have].c_attrs = ATR_NORMAL;
+		c[have].c_value = SPACE;
+		c[have].c_level = 0;
+		c[have].c_ident = CS_NORMAL;
+		have++;
+	    }
+	    cur_line->l_last = want;
+	    cur_line->l_cell = c;
 	}
-	cur_line->l_last = want;
-	cur_line->l_cell = c;
     }
 }
 
@@ -303,8 +399,8 @@ extend_line(void)
 static void
 put_cell(int c, int level, int ident, int attrs)
 {
-    int col;
-    int len;
+    size_t col;
+    size_t len;
     CHARCELL *p, *q;
 
     if (cur_line == 0)
@@ -330,89 +426,304 @@ put_cell(int c, int level, int ident, int attrs)
 		q = allocate_cell();
 		*q = *p;
 		p->link = q;
-	    } else if (c == SPACE)
-		return;
+	    }
 	}
     }
 
-    p->c_value = c;
-    p->c_level = level;
-    p->c_ident = ident;
+    p->c_value = (unsigned) c;
+    p->c_level = (char) level;
+    p->c_ident = (char) ident;
 
     if (cur_line->l_used < cur_line->l_this)
 	cur_line->l_used = cur_line->l_this;
 }
 
+#define MAXPARAMS 10
+
+static void
+erase_cell(int col)
+{
+    /* *INDENT-EQLS* */
+    CHARCELL *p = &(cur_line->l_cell[col]);
+    p->link     = 0;
+    p->c_value  = SPACE;
+    p->c_level  = 0;
+    p->c_ident  = CS_NORMAL;
+}
+
+#define DefaultOne(nparams, params) \
+	((nparams == 0) ? 1 : (nparams == 1) ? params[0] : -1)
+
+#define DefaultZero(nparams, params) \
+	((nparams == 0) ? 0 : (nparams == 1) ? params[0] : -1)
+
+static void
+ansi_CUB(int code)
+{
+    if (code > 0 && ((int) cur_line->l_this - code) >= 0) {
+	cur_line->l_this = cur_line->l_this - (size_t) code;
+    }
+}
+
+static void
+ansi_CUF(int code)
+{
+    if (code > 0) {
+	size_t col = cur_line->l_this + (size_t) code;
+	while (col > cur_line->l_last) {
+	    extend_line();
+	}
+	if (cur_line->l_used < col)
+	    cur_line->l_used = col;
+	cur_line->l_this = col;
+    }
+}
+
+static void
+ansi_DCH(int code)
+{
+    size_t dst;
+    size_t src;
+
+    if (code > 0) {
+	for (dst = cur_line->l_this; dst < cur_line->l_used; ++dst) {
+	    src = dst + (size_t) code;
+	    if (src < cur_line->l_used) {
+		cur_line->l_cell[dst] = cur_line->l_cell[src];
+	    } else {
+		erase_cell((int) dst);
+	    }
+	}
+	cur_line->l_used -= (size_t) code;
+    }
+}
+
+static void
+ansi_EL(int code)
+{
+    size_t col;
+
+    switch (code) {
+    case 0:			/* Erase to Right (default) */
+	for (col = cur_line->l_this; col <= cur_line->l_used; ++col)
+	    erase_cell((int) col);
+	break;
+    case 1:			/* Erase to Left */
+    case 2:			/* Erase All */
+	for (col = 0; col <= cur_line->l_this; ++col)
+	    erase_cell((int) col);
+	break;
+    }
+}
+
+static void
+ansi_ICH(int code)
+{
+    size_t dst;
+    size_t src;
+
+    if (code > 0) {
+	size_t last = cur_line->l_last - 1;
+	for (dst = last; dst >= cur_line->l_this; --dst) {
+	    src = dst - (size_t) code;
+	    if (src >= cur_line->l_this) {
+		cur_line->l_cell[dst] = cur_line->l_cell[src];
+	    } else {
+		erase_cell((int) dst);
+	    }
+	}
+	if (cur_line->l_used < last)
+	    cur_line->l_used++;
+    }
+}
+
+static void
+ansi_HPA(int code)
+{
+    if (code > 0) {
+	size_t col = (size_t) code - 1;
+	while (col > cur_line->l_last) {
+	    extend_line();
+	}
+	if (cur_line->l_used < col)
+	    cur_line->l_used = col;
+	cur_line->l_this = col;
+    }
+}
+
 /*
  * Interpret equivalent overstrike/underline for an ANSI escape sequence.
  */
 static int
 ansi_escape(FILE *ifp, int last_code)
 {
-    int ansi = 1;
-    int code = ATR_NORMAL;
+    int code = last_code;
+    int final = 0;
     int value = 0;
     int digits = 0;
+    int private = 0;
+    int params[MAXPARAMS + 1];
+    int nparams = 0;
     int c;
 
     while ((c = my_getc(ifp)) != EOF) {
-	c = CharOf(c);
-	if (isdigit(c)) {
+	if (c >= 0x30 && c <= 0x39) {
 	    value = (value * 10) + (c - '0');
 	    digits++;
-	} else {
-	    if (digits) {
-		switch (value) {
-		case 1:
-		    code |= ATR_BOLD;
-		    break;
-		case 3:
-		    code |= ATR_ITAL;
-		    break;
-		case 4:
-		    code |= ATR_UNDER;
-		    break;
-		case 7:
-		    code |= ATR_REVERS;
-		    break;
-		case 22:
-		    code &= ~ATR_BOLD;
-		    break;
-		case 23:
-		    code &= ~ATR_BOLD;
-		    break;
-		case 24:
-		    code &= ~ATR_UNDER;
-		    break;
-		case 27:
-		    code &= ~ATR_REVERS;
-		    break;
-		default:
-		    /* we handle only foreground (text) colors */
-		    if (value >= 30 && value <= 37) {
-			value -= 30;
-			value <<= SHL_COLOR;
-			code &= ~ATR_COLOR;
-			code |= value;
-		    }
-		    break;
+	} else if (c == ';' || (c >= 0x40 && c <= 0x7e)) {
+	    if (nparams < MAXPARAMS) {
+		if (!digits) {
+		    value = -1;
+		} else {
+		    params[nparams++] = value;
 		}
 	    }
-	    if (isalpha(c))
+	    if (c != ';') {
+		final = c;
 		break;
-	    else if (c != ';')
-		ansi = 0;
+	    }
 	    value = 0;
 	    digits = 0;
+	} else if (c >= 0x3a && c <= 0x3f) {
+	    private = 1;
+	} else if (c < 0x30 || c > 0x7e) {
+	    break;		/* ...at least, nothing we can handle */
 	}
     }
 
-    if ((c != 'm') || !ansi) {
-	code = last_code;
+    if (!private) {
+	/*
+	 * Multiline controls are implemented (but are noted below) because
+	 * doing this correctly depends on the filter knowing the width of
+	 * the terminal when the log/typescript was produced.  That width was
+	 * taken into account by the program which wrote the log, and is not
+	 * necessarily the same as that used in the current session.
+	 */
+	switch (final) {
+	case '@':
+	    ansi_ICH(DefaultOne(nparams, params));
+	    break;
+	case 'A':
+	    /* CUU - not implemented (multiline) */
+	    break;
+	case 'B':
+	    /* CUD - not implemented (multiline) */
+	    break;
+	case 'C':
+	    ansi_CUF(DefaultOne(nparams, params));
+	    break;
+	case 'D':
+	    ansi_CUB(DefaultOne(nparams, params));
+	    break;
+	case '`':
+	case 'G':
+	    ansi_HPA(DefaultOne(nparams, params));
+	    break;
+	case 'H':
+	    /* CUP - not implemented (multiline) */
+	    break;
+	case 'J':
+	    /* ED - not implemented (multiline) */
+	    break;
+	case 'K':
+	    ansi_EL(DefaultZero(nparams, params));
+	    break;
+	case 'L':
+	    /* IL - not implemented (multiline) */
+	    break;
+	case 'M':
+	    /* DL - not implemented (multiline) */
+	    break;
+	case 'P':
+	    ansi_DCH(DefaultOne(nparams, params));
+	    break;
+	case 'd':
+	    /* VPA - not implemented (multiline) */
+	    break;
+	case 'h':
+	case 'l':
+	    /* 4 is IRM - not implemented (ICH is preferred) */
+	    break;
+	case 'm':
+	    /* SGR */
+	    if (nparams != 0) {
+		for (c = 0; c < nparams; ++c) {
+		    value = params[c];
+		    switch (value) {
+		    case 0:
+			code = ATR_NORMAL;
+			break;
+		    case 1:
+			code |= ATR_BOLD;
+			break;
+		    case 3:
+			code |= ATR_ITAL;
+			break;
+		    case 4:
+			code |= ATR_UNDER;
+			break;
+		    case 7:
+			code |= ATR_REVERS;
+			break;
+		    case 22:
+			code &= ~ATR_BOLD;
+			break;
+		    case 23:
+			code &= ~ATR_BOLD;
+			break;
+		    case 24:
+			code &= ~ATR_UNDER;
+			break;
+		    case 27:
+			code &= ~ATR_REVERS;
+			break;
+		    default:
+			/* we handle only foreground (text) colors */
+			if (value >= 30 && value <= 37) {
+			    value -= 30;
+			    value <<= SHL_COLOR;
+			    code &= ~ATR_COLOR;
+			    code |= value;
+			} else if (value >= 90 && value <= 97) {
+			    value -= 90;
+			    value += 8;
+			    value <<= SHL_COLOR;
+			    code &= ~ATR_COLOR;
+			    code |= value;
+			}
+			break;
+		    }
+		}
+	    } else {
+		code = ATR_NORMAL;
+	    }
+	    break;
+	}
     }
     return code;
 }
 
+/*
+ * Simply ignore OSC strings, e.g., as used to set window title.
+ * This does not actually wait for a string terminator, since we may be stuck
+ * with some Linux console junk.
+ */
+static void
+osc_ignored(FILE *ifp)
+{
+    int c;
+
+    while ((c = my_getc(ifp)) != EOF) {
+	if (c == ESCAPE) {
+	    c = my_getc(ifp);
+	    break;
+	} else if (c < 0x20 || ((c >= 0x80) && (c < 0xa0))) {
+	    break;
+	}
+    }
+
+}
+
 /*
  * Set the current pointer to the previous line, allocating it if necessary
  */
@@ -506,6 +817,7 @@ static void
 flush_line(void)
 {
     size_t col;
+    size_t ccol;
     int ref_code;
     int tst_code;
     int counter;
@@ -524,35 +836,35 @@ flush_line(void)
 		tst_code = cell_code(l, col);
 		if (tst_code != ref_code) {
 		    ref_code = tst_code;
-		    for (counter = 1; counter + col < l->l_used; counter++) {
-			tst_code = cell_code(l, col + counter);
+		    for (ccol = col + 1; ccol < l->l_used; ccol++) {
+			tst_code = cell_code(l, ccol);
 			if (tst_code != ref_code)
 			    break;
 		    }
 		    if (ref_code != ATR_NORMAL) {
-			printf("%c%d", CNTL_A, counter);
+			printf("%c%u", CNTL_A, (unsigned) (ccol - col));
 			if (ref_code & ATR_BOLD)
-			    putchar('B');
+			    my_putc('B');
 			if (ref_code & ATR_ITAL)
-			    putchar('I');
+			    my_putc('I');
 			if (ref_code & ATR_UNDER)
-			    putchar('U');
+			    my_putc('U');
 			if (ref_code & ATR_REVERS)
-			    putchar('R');
+			    my_putc('R');
 			if (ref_code & ATR_COLOR)
 			    printf("C%X", (ref_code >> SHL_COLOR) & 0xf);
-			putchar(':');
+			my_putc(':');
 		    }
 		}
 	    }
-	    putchar(l->l_cell[col].c_value);
+	    my_putc((int) l->l_cell[col].c_value);
 
 	    while ((p = l->l_cell[col].link) != 0) {
 		l->l_cell[col].link = p->link;
 		free((char *) p);
 	    }
 	}
-	putchar('\n');
+	my_putc('\n');
 
 	if (l != 0) {
 	    if (l->l_cell != 0)
@@ -574,7 +886,6 @@ ManFilter(FILE *ifp)
     int esc_mode = ATR_NORMAL;
 
     while ((c = my_getc(ifp)) != EOF) {
-	c = CharOf(c);
 	switch (c) {
 	case '\b':
 	    backspace();
@@ -613,6 +924,9 @@ ManFilter(FILE *ifp)
 	    case '[':
 		esc_mode = ansi_escape(ifp, esc_mode);
 		break;
+	    case ']':
+		osc_ignored(ifp);
+		break;
 	    case '\007':
 	    case '7':
 		prev_line();
@@ -630,12 +944,19 @@ ManFilter(FILE *ifp)
 	    }
 	    break;
 
+	case 0xb7:		/* bullet character */
+	    if (!sys_isprint(c)) {
+		c = 'o';
+	    }
+	    goto printable;
+
 	case 0xad:		/* mis-used softhyphen */
 	    c = '-';
 	    /* FALLTHRU */
 
+	  printable:
 	default:		/* ignore other nonprinting characters */
-	    if (isprint(c)) {
+	    if (sys_isprint(c)) {
 		put_cell(c, level, ident, esc_mode);
 		if (c != SPACE) {
 		    if (esc_mode & ATR_BOLD) {
@@ -662,8 +983,10 @@ int
 main(int argc, char **argv)
 {
     int n;
+    FILE *fp;
 
     my_getc = ansi_getc;
+    my_putc = ansi_putc;
 #if OPT_LOCALE
     {
 	char *env;
@@ -672,9 +995,12 @@ main(int argc, char **argv)
 	    ((env = getenv("LC_CTYPE")) != 0 && *env != 0) ||
 	    ((env = getenv("LANG")) != 0 && *env != 0)) {
 
-	    if (strstr(env, ".UTF-8") != 0) {
-		utf8_locale = 1;
+	    if (strstr(env, ".UTF-8") != 0
+		|| strstr(env, ".utf-8") != 0
+		|| strstr(env, ".UTF8") != 0
+		|| strstr(env, ".utf8") != 0) {
 		my_getc = utf8_getc;
+		my_putc = utf8_putc;
 	    }
 	}
 	setlocale(LC_CTYPE, "");
@@ -683,11 +1009,17 @@ main(int argc, char **argv)
 
     if (argc > 1) {
 	for (n = 1; n < argc; n++) {
-	    FILE *fp = fopen(argv[n], "r");
-	    if (fp == 0)
-		failed(argv[n]);
-	    ManFilter(fp);
-	    (void) fclose(fp);
+	    char *param = argv[n];
+	    if (*param == '-') {
+		if (!strcmp(param, "-8"))
+		    opt_8bit = 1;
+	    } else {
+		fp = fopen(argv[n], "r");
+		if (fp == 0)
+		    failed(argv[n]);
+		ManFilter(fp);
+		(void) fclose(fp);
+	    }
 	}
     } else {
 	ManFilter(stdin);
