@@ -5,7 +5,7 @@
  *	included in main.c
  *
  *	Copyright (c) 1990 by Paul Fox
- *	Copyright (c) 1995-2004 by Paul Fox and Thomas Dickey
+ *	Copyright (c) 1995-2015 by Paul Fox and Thomas Dickey
  *
  *	See the file "cmdtbl" for input data formats, and "estruct.h" for
  *	the output structures.
@@ -15,16 +15,10 @@
  * by Tom Dickey, 1993.    -pgf
  *
  *
- * $Header: /usr/build/vile/vile/RCS/mktbls.c,v 1.132 2004/12/03 00:01:22 tom Exp $
+ * $Header: /usr/build/vile/vile/RCS/mktbls.c,v 1.191 2015/02/01 21:33:42 tom Exp $
  *
  */
 
-#if defined(__OS2__) && defined(__IBMC__) && (__IBMC__ >= 200)
-#define USE_OFFSETS 0
-#else
-#define USE_OFFSETS 1
-#endif
-
 /* stuff borrowed/adapted from estruct.h */
 
 #ifdef HAVE_CONFIG_H
@@ -44,10 +38,18 @@
 
 #endif /* !defined(HAVE_CONFIG_H) */
 
+#if defined(WIN32)
+#include "w32vile.h"
+#endif
+
 #ifndef SYS_VMS
 #define SYS_VMS 0
 #endif
 
+#ifndef HAVE_CONFIG_H
+# define HAVE_STDLIB_H 1
+#endif
+
 #ifndef HAVE_STDLIB_H
 # define HAVE_STDLIB_H 0
 #endif
@@ -64,6 +66,11 @@ extern void free(char *ptr);
 #endif
 
 /*----------------------------------------------------------------------------*/
+#include <stdio.h>
+#include <ctype.h>
+#include <string.h>
+#include <setjmp.h>
+
 #ifndef DOALLOC
 #define DOALLOC 0
 #endif
@@ -72,14 +79,14 @@ extern void free(char *ptr);
 #include "trace.h"
 #endif
 
+#ifndef GCC_NORETURN
+#define GCC_NORETURN		/*nothing */
+#endif
+
 #ifndef NO_LEAKS
 #define NO_LEAKS 0
 #endif
 
-#include <stdio.h>
-#include <string.h>
-#include <setjmp.h>
-
 /* argument for 'exit()' or '_exit()' */
 #if	SYS_VMS
 #include	<stsdef.h>
@@ -90,6 +97,8 @@ extern void free(char *ptr);
 #define BADEXIT		1
 #endif
 
+#define ENUM_PREFIX	"vile_"
+
 #define	TABLESIZE(v)	(sizeof(v)/sizeof(v[0]))
 
 #ifndef OPT_EXEC_MACROS
@@ -100,7 +109,7 @@ extern void free(char *ptr);
 
 #define MAX_BIND        4	/* maximum # of key-binding types */
 #define	MAX_PARSE	5	/* maximum # of tokens on line */
-#define	LEN_BUFFER	60	/* nominal buffer-length */
+#define	LEN_BUFFER	220	/* nominal buffer-length */
 #define	MAX_BUFFER	(LEN_BUFFER*10)
 #define	LEN_CHRSET	256	/* total # of chars in set (ascii) */
 
@@ -108,11 +117,8 @@ extern void free(char *ptr);
 #define	DIFCNTRL	0x40
 #define tocntrl(c)	((c)^DIFCNTRL)
 #define toalpha(c)	((c)^DIFCNTRL)
-#define	DIFCASE		0x20
 #define	isUpper(c)	((c) >= 'A' && (c) <= 'Z')
 #define	isLower(c)	((c) >= 'a' && (c) <= 'z')
-#define toUpper(c)	((c)^DIFCASE)
-#define toLower(c)	((c)^DIFCASE)
 #define isboolean(c)	((c) == 'b' || (c) == 'M')
 
 #ifndef	TRUE
@@ -144,12 +150,15 @@ typedef struct stringl {
     char *Data;			/* associated data, if any */
     char *Cond;			/* stores ifdef-flags */
     char *Note;			/* stores comment, if any */
+    char *Flag;			/* stores execution flags, if any */
     struct stringl *nst;
 } LIST;
 
-static char *Blank = "";
+static char Blank[] =
+{0};
 
 static LIST *all_names;
+static LIST *all_aliases;
 static LIST *all_kbind;		/* data for kbindtbl[] */
 static LIST *all_w32bind;	/* w32 data for kbindtbl[] */
 static LIST *all_funcs;		/* data for extern-lines in neproto.h */
@@ -167,8 +176,9 @@ static LIST *all_qmodes;	/* data for QUALIFIER modes */
 static LIST *all_bmodes;	/* data for BUFFER modes */
 static LIST *all_wmodes;	/* data for WINDOW modes */
 
+static void badfmt(const char *) GCC_NORETURN;
 static void save_all_modes(const char *type, char *normal, const char
-			   *abbrev, char *cond);
+			   *abbrev, const char *cond);
 static void save_all_submodes(const char *type, char *normal, const char
 			      *abbrev, char *cond);
 
@@ -183,6 +193,7 @@ typedef enum {
     ,SECT_BUFF
     ,SECT_WIND
     ,SECT_FSMS
+    ,SECT_ABBR
 } SECTIONS;
 
 	/* definitions for indices to 'asciitbl[]' vs 'kbindtbl[]' */
@@ -210,7 +221,8 @@ static const char *prefname[MAX_BIND] =
     "SPEC|"
 };
 
-static char *fsm_name;
+static char *fsm_uc_name;
+static char *fsm_lc_name;
 static char *inputfile;
 static int l = 0;
 static FILE *cmdtbl;
@@ -226,6 +238,56 @@ static FILE *nefsms;
 static jmp_buf my_top;
 
 /******************************************************************************/
+static char *
+my_strncpy0(char *dest, const char *src, size_t destlen)
+{
+    if (dest != 0 && src != 0 && destlen != 0) {
+	(void) strncpy(dest, src, destlen);
+	dest[destlen - 1] = EOS;
+    }
+    return dest;
+}
+
+/*
+ * This is probably more efficient for copying short strings into a large
+ * fixed-size buffer, because strncpy always zero-pads the destination to
+ * the given length.
+ */
+static char *
+my_strncpy(char *dest, const char *src, size_t destlen)
+{
+    size_t srclen = (src != 0) ? (strlen(src) + 1) : 0;
+    if (srclen > destlen)
+	srclen = destlen;
+    return my_strncpy0(dest, src, srclen);
+}
+
+static char *
+my_strncat(char *dest, const char *src, size_t destlen)
+{
+    size_t srclen = (src != 0) ? (strlen(src) + 1) : 0;
+    size_t oldlen = strlen(dest);
+
+    if (srclen > (destlen - oldlen))
+	srclen = (destlen - oldlen);
+
+    (void) my_strncpy0(dest + oldlen, src, srclen);
+    return dest;
+}
+
+/******************************************************************************/
+static int
+toUpper(int c)
+{
+    return toupper((c) & 0xff);
+}
+
+static int
+toLower(int c)
+{
+    return tolower((c) & 0xff);
+}
+
 static int
 isSpace(int c)
 {
@@ -238,6 +300,17 @@ isPrint(int c)
     return c >= ' ' && c < 0x7f;
 }
 
+static char *
+lowercase(char *s)
+{
+    char *base = s;
+    while (*s != 0) {
+	*s = (char) toLower(*s);
+	++s;
+    }
+    return base;
+}
+
 /******************************************************************************/
 static void
 badfmt(const char *s)
@@ -256,10 +329,10 @@ badfmt2(const char *s, int col)
 }
 
 /******************************************************************************/
-static char *
-Alloc(unsigned len)
+static void *
+Alloc(size_t len)
 {
-    char *pointer = (char *) malloc(len);
+    void *pointer = calloc(len, sizeof(char));
     if (pointer == 0)
 	badfmt("bug: not enough memory");
     return pointer;
@@ -268,7 +341,7 @@ Alloc(unsigned len)
 static char *
 StrAlloc(const char *s)
 {
-    return strcpy(Alloc((unsigned) strlen(s) + 1), s);
+    return strcpy(Alloc(strlen(s) + 1), s);
 }
 
 static LIST *
@@ -294,8 +367,11 @@ free_LIST(LIST ** p)
 	    FreeIfNeeded(q->Cond);
 	if (q->Note != Blank)
 	    FreeIfNeeded(q->Note);
+	if (q->Flag != 0)
+	    FreeIfNeeded(q->Flag);
 	free((char *) q);
     }
+    *p = 0;
 }
 
 /******************************************************************************/
@@ -332,7 +408,8 @@ InsertSorted(
 		const char *func,
 		const char *data,
 		const char *cond,
-		const char *note)
+		const char *note,
+		const char *flag)
 {
     LIST *n, *p, *q;
     int r;
@@ -343,6 +420,7 @@ InsertSorted(
     n->Data = StrAlloc(data);
     n->Cond = StrAlloc(cond);
     n->Note = StrAlloc(note);
+    n->Flag = StrAlloc(flag);
 
     for (p = *headp, q = 0; p != 0; q = p, p = p->nst) {
 	if ((r = strcmp(n->Name, p->Name)) < 0)
@@ -419,7 +497,9 @@ LastCol(char *buffer)
 static char *
 PadTo(int col, char *buffer)
 {
-    int any = 0, len = strlen(buffer), now;
+    int any = 0;
+    int len = (int) strlen(buffer);
+    int now;
     char with;
 
     for (;;) {
@@ -469,7 +549,7 @@ set_binding(int btype, int c, char *cond, char *func)
 		    (c == '\'' || c == '\\') ? "\\" : "",
 		    c);
 	}
-	InsertSorted(&all_kbind, name, func, "", cond, "");
+	InsertSorted(&all_kbind, name, func, "", cond, "", "");
     } else {
 	if (bindings[c] != NULL) {
 	    if (!two_conds(c, cond))
@@ -496,8 +576,8 @@ Parse(char *input, char **vec)
     int expecting = TRUE, count = 0, quote = 0, n, c;
 
     for (c = 0; c < MAX_PARSE; c++)
-	vec[c] = "";
-    for (c = strlen(input); c > 0 && isSpace(input[c - 1]); c--)
+	vec[c] = Blank;
+    for (c = (int) strlen(input); c > 0 && isSpace(input[c - 1]); c--)
 	input[c - 1] = EOS;
 
     for (n = 0; (c = input[n++]) != EOS;) {
@@ -556,7 +636,7 @@ WriteIf(FILE *fp, const char *cond)
 {
     if (cond == 0)
 	cond = "";
-    if (cond[0] != EOS) {
+    if (cond[0] != EOS && strcmp(cond, "1")) {
 	if (lastIfdef != 0) {
 	    if (!strcmp(lastIfdef, cond))
 		return;
@@ -642,17 +722,26 @@ is_majormode(const char *name)
 static void
 CheckModes(char *name)
 {
-    if (!strncmp(name, "no", 2))
+    if (!strncmp(name, "no", (size_t) 2))
 	badfmt("illegal mode-name");
 }
 
+static char *
+AllocKey(char *normal, int type, char *abbrev)
+{
+    char *tmp = Alloc(4 + strlen(normal) + strlen(abbrev));
+    Sprintf(tmp, "%s\n%c\n%s", normal, type, abbrev);
+    return tmp;
+}
+
 /* make a sort-key for mode-name */
 static char *
 Mode2Key(char *type, char *name, char *cond, int submode)
 {
     int c;
-    char *abbrev = AbbrevMode(name), *normal = NormalMode(name), *tmp =
-    Alloc((unsigned) (4 + strlen(normal) + strlen(abbrev)));
+    char *abbrev = AbbrevMode(name);
+    char *normal = NormalMode(name);
+    char *tmp;
 
     CheckModes(normal);
     CheckModes(abbrev);
@@ -671,7 +760,45 @@ Mode2Key(char *type, char *name, char *cond, int submode)
     if (submode)
 	save_all_submodes(type, normal, abbrev, cond);
 
-    Sprintf(tmp, "%s\n%c\n%s", normal, c, abbrev);
+    tmp = AllocKey(normal, c, abbrev);
+#if NO_LEAKS
+    free(normal);
+    free(abbrev);
+#endif
+    return tmp;
+}
+
+/* make a sort-key for state-variables */
+static char *
+Vars2Key(char *type, char *name, char *cond)
+{
+    int c;
+    char *abbrev = AbbrevMode(name);
+    char *normal = NormalMode(name);
+    char *tmp;
+
+    CheckModes(normal);
+    CheckModes(abbrev);
+
+    switch (c = *type) {
+    case 'b':
+    case 'e':
+    case 'i':
+    case 's':
+	break;
+    case 'r':
+	c = 'x';		/* make this sort after strings */
+    }
+
+    /* insert into 'all_modes' to provide for common name-completion
+     * table, and into 'all_statevars' to get name/index correspondence.
+     */
+    InsertSorted(&all_modes, name, "state", "",
+		 formcond("OPT_EVAL", cond),
+		 "",
+		 "");
+
+    tmp = AllocKey(normal, c, abbrev);
 #if NO_LEAKS
     free(normal);
     free(abbrev);
@@ -688,7 +815,7 @@ Name2Symbol(char *name)
 
     /* allocate enough for adjustment in 'Name2Address()' */
     /*   "+ 10" for comfort */
-    base = dst = Alloc((unsigned) (strlen(name) + 10));
+    base = dst = Alloc(strlen(name) + 10);
 
     *dst++ = 's';
     *dst++ = '_';
@@ -708,14 +835,15 @@ static char *
 Name2Address(char *name, char *type)
 {
     /*  "+ 10" for comfort */
-    unsigned len = strlen(name) + 10;
-    char *base = Alloc(len);
+    size_t len = strlen(name) + 10;
+    char *base;
     char *temp;
 
     temp = Name2Symbol(name);
-    if (strlen(temp) + 1 + (isboolean(*type) ? 4 : 0) > len)
+    if (strlen(temp) + 1 + (size_t) (isboolean(*type) ? 4 : 0) > len)
 	badfmt("bug: buffer overflow in Name2Address");
 
+    base = Alloc(len);
     (void) strcpy(base, temp);
     if (isboolean(*type))
 	(void) strcat(strcat(strcpy(base + 2, "no"), temp + 2), "+2");
@@ -723,61 +851,24 @@ Name2Address(char *name, char *type)
     return base;
 }
 
-/* define Member_Offset macro, used in index-definitions */
-static void
-DefineOffset(FILE *fp)
-{
-#if USE_OFFSETS
-    Fprintf(fp,
-	    "#ifndef\tMember_Offset\n\
-#define\tMember_Offset(T,M) ((int)(((long)&(((T*)0)->M))/\\\n\
-\t\t\t\t ((long)&(((T*)0)->Q1) - (long)&(((T*)0)->s_MAX))))\n\
-#endif\n");
-#else
-    Fprintf(fp,
-	    "#ifndef\tMember_Offset\n\
-#define\tMember_Offset(T,M) ((vile_ ## M)-1)\n\
-#endif\n");
-#endif
-}
-
 /* generate the index-struct (used for deriving ifdef-able index definitions) */
 static void
 WriteIndexStruct(FILE *fp, LIST * p, const char *ppref)
 {
-#if USE_OFFSETS
-    char *s, temp[MAX_BUFFER], line[MAX_BUFFER], *vec[MAX_PARSE];
-
-    BeginIf();
-    Fprintf(fp, "typedef\tstruct\t%c\n", L_CURL);
-    while (p != 0) {
-	WriteIf(fp, p->Cond);
-	(void) Parse(strcpy(line, p->Name), vec);
-	Sprintf(temp, "\tchar\t%s;", s = Name2Symbol(vec[1]));
-	free(s);
-	if (p->Note[0]) {
-	    (void) PadTo(32, temp);
-	    Sprintf(temp + strlen(temp), "/* %s */", p->Note);
-	}
-	Fprintf(fp, "%s\n", temp);
-	p = p->nst;
-    }
-
-    FlushIf(fp);
-    Fprintf(fp, "\tchar\ts_MAX;\n");
-    Fprintf(fp, "\tchar\tQ1;\n");
-    Fprintf(fp, "\t%c Index%s;\n\n", R_CURL, ppref);
-#else
     char *s, temp[MAX_BUFFER], line[MAX_BUFFER], *vec[MAX_PARSE];
     int count = 0;
 
     BeginIf();
     Fprintf(fp, "typedef\tenum\t%c\n", L_CURL);
-    Fprintf(fp, "\tMIN_%c_VALUES = 0\n", *ppref);
+    /*
+     * The first symbol is unused; it is a placeholder in case the first value
+     * is ifdef'd out.
+     */
+    Fprintf(fp, "\tDUMMY_%c_VALUES = -1\n", *ppref);
     for (; p != 0; p = p->nst, count++) {
 	WriteIf(fp, p->Cond);
-	(void) Parse(strcpy(line, p->Name), vec);
-	Sprintf(temp, "\t,vile_%s", s = Name2Symbol(vec[1]));
+	(void) Parse(my_strncpy(line, p->Name, sizeof(line)), vec);
+	Sprintf(temp, "\t,%s%s", ENUM_PREFIX, s = Name2Symbol(vec[1]));
 	free(s);
 	if (p->Note[0]) {
 	    (void) PadTo(32, temp);
@@ -794,7 +885,6 @@ WriteIndexStruct(FILE *fp, LIST * p, const char *ppref)
     Fprintf(nemode, "%s", temp);
 
     Fprintf(fp, "\t%c Index%s;\n\n", R_CURL, ppref);
-#endif
 }
 
 /* generate the index-definitions */
@@ -807,15 +897,15 @@ WriteModeDefines(LIST * p, const char *ppref)
     BeginIf();
 
     for (; p != 0; p = p->nst, count++) {
-	(void) Parse(strcpy(line, p->Name), vec);
+	(void) Parse(my_strncpy(line, p->Name, sizeof(line)), vec);
 	Sprintf(temp, "#define %.1s%s%s",
 		(*ppref == 'B') ? "" : ppref,
 		(*vec[2] == 'b') ? "MD" : "VAL_",
 		p->Func);
 	(void) PadTo(24, temp);
 	WriteIf(nemode, p->Cond);
-	Sprintf(temp + strlen(temp), "Member_Offset(Index%s, %s)",
-		ppref, s = Name2Symbol(vec[1]));
+	Sprintf(temp + strlen(temp), "%s%s",
+		ENUM_PREFIX, s = Name2Symbol(vec[1]));
 	free(s);
 	Fprintf(nemode, "%s\n", temp);
     }
@@ -823,15 +913,6 @@ WriteModeDefines(LIST * p, const char *ppref)
     Fprintf(nemode, "\n");
     FlushIf(nemode);
 
-#if USE_OFFSETS
-    Sprintf(temp, "#define NUM_%c_VALUES\tMember_Offset(Index%s, s_MAX)",
-	    *ppref, ppref);
-    (void) PadTo(32, temp);
-    Sprintf(temp + strlen(temp), "/* TABLESIZE(%c_valnames) -- %s */\n",
-	    toLower(*ppref), ppref);
-    Fprintf(nemode, "%s", temp);
-#endif
-
     Fprintf(nemode, "#define MAX_%c_VALUES\t%d\n\n", *ppref, count);
 }
 
@@ -845,7 +926,7 @@ WriteModeSymbols(LIST * p)
     BeginIf();
     while (p != 0) {
 	WriteIf(nemode, p->Cond);
-	(void) Parse(strcpy(line, p->Name), vec);
+	(void) Parse(my_strncpy(line, p->Name, sizeof(line)), vec);
 	Sprintf(temp, "\t%c %s,",
 		L_CURL, s = Name2Address(vec[1], vec[2]));
 	(void) PadTo(32, temp);
@@ -853,7 +934,9 @@ WriteModeSymbols(LIST * p)
 	s = 0;
 
 	Sprintf(temp + strlen(temp), "%s,",
-		*vec[3] ? (s = Name2Address(vec[3], vec[2])) : "\"X\"");
+		(*vec[3]
+		 ? (s = Name2Address(vec[3], vec[2]))
+		 : "\"X\""));
 	(void) PadTo(48, temp);
 	if (s != 0)
 	    free(s);
@@ -861,14 +944,13 @@ WriteModeSymbols(LIST * p)
 	Sprintf(temp + strlen(temp), "VALTYPE_%s,", c2TYPE(*vec[2]));
 	(void) PadTo(64, temp);
 	if (!strcmp(p->Data, "0"))
-	    (void) strcat(temp, "(ChgdFunc)0 },");
+	    (void) my_strncat(temp, "(ChgdFunc)0 },", sizeof(temp));
 	else
 	    Sprintf(temp + strlen(temp), "%s },", p->Data);
 	Fprintf(nemode, "%s\n", temp);
 	p = p->nst;
     }
     FlushIf(nemode);
-
 }
 
 /******************************************************************************/
@@ -877,20 +959,22 @@ save_all_modes(
 		  const char *type,
 		  char *normal,
 		  const char *abbrev,
-		  char *cond)
+		  const char *cond)
 {
     if (isboolean(*type)) {
 	char t_normal[LEN_BUFFER], t_abbrev[LEN_BUFFER];
-	save_all_modes("Bool",
-		       strcat(strcpy(t_normal, "no"), normal),
-		       *abbrev
-		       ? strcat(strcpy(t_abbrev, "no"), abbrev)
-		       : "",
-		       cond);
+	my_strncat(strcpy(t_normal, "no"), normal, sizeof(t_normal));
+	if (*abbrev) {
+	    my_strncat(strcpy(t_abbrev, "no"), abbrev, sizeof(t_abbrev));
+	} else {
+	    strcpy(t_abbrev, "");
+	}
+	save_all_modes("Bool", t_normal, t_abbrev, cond);
+    }
+    InsertSorted(&all_modes, normal, type, "", cond, "", "");
+    if (*abbrev && strcmp(normal, abbrev)) {
+	InsertSorted(&all_modes, abbrev, type, "", cond, "", "");
     }
-    InsertSorted(&all_modes, normal, type, "", cond, "");
-    if (*abbrev)
-	InsertSorted(&all_modes, abbrev, type, "", cond, "");
 }
 
 static void
@@ -924,7 +1008,7 @@ dump_all_modes(void)
     char temp[MAX_BUFFER], *s;
     LIST *p, *q;
 
-    InsertSorted(&all_modes, "all", "?", "", "", "");
+    InsertSorted(&all_modes, "all", "?", "", "", "", "");
     write_lines(nemode, top);
     BeginIf();
     for (p = all_modes; p; p = p->nst) {
@@ -1087,7 +1171,7 @@ dump_majors(void)
     LIST *p, *q;
     char norm[LEN_BUFFER], abbr[LEN_BUFFER], type[LEN_BUFFER];
     char normal[LEN_BUFFER], abbrev[LEN_BUFFER];
-    char *my_cond = "OPT_MAJORMODE";
+    const char *my_cond = "OPT_MAJORMODE";
 
     for (q = all_majors; q; q = q->nst) {
 	Sprintf(normal, "%smode", q->Name);	/* FIXME */
@@ -1102,6 +1186,7 @@ dump_majors(void)
 	    save_all_modes(c2TYPE(*type), normal, abbrev, my_cond);
 	}
     }
+    free_LIST(&all_majors);
 }
 
 /******************************************************************************/
@@ -1110,7 +1195,7 @@ save_mmodes(char *type, char **vec)
 {
     char *key = Mode2Key(type, vec[1], vec[4], TRUE);
 
-    InsertSorted(&all_mmodes, key, vec[2], vec[3], vec[4], vec[0]);
+    InsertSorted(&all_mmodes, key, vec[2], vec[3], vec[4], vec[0], "");
 #if NO_LEAKS
     free(key);
 #endif
@@ -1156,6 +1241,55 @@ dump_mmodes(void)
     write_lines(nemode, bottom);
 }
 
+/******************************************************************************/
+static void
+save_abbr_in(LIST ** q, LIST * p, char **vec)
+{
+    char full_name[LEN_BUFFER];
+    char part_name[LEN_BUFFER];
+    char type_name[LEN_BUFFER];
+    int count;
+
+    for (; p; p = p->nst) {
+	*full_name = EOS;
+	*type_name = EOS;
+	*part_name = EOS;
+	count = sscanf(p->Name, "%s\n%s\n%s", full_name, type_name, part_name);
+	if (count == 2 || (count == 3 && *part_name == EOS)) {
+	    if (!strcmp(full_name, vec[1])) {
+		free(p->Name);
+		p->Name = AllocKey(full_name, *type_name, vec[2]);
+		*q = p;
+	    }
+	} else if (count == 3) {
+	    if (!strcmp(full_name, vec[1])) {
+		fprintf(stderr, "\"%s\" already has alias \"%s\"\n",
+			full_name,
+			vec[2]);
+		badfmt("naming conflict");
+	    }
+	}
+    }
+}
+
+static void
+save_abbrs(char **vec)
+{
+    LIST *p = 0;
+
+    save_abbr_in(&p, all_gmodes, vec);
+    save_abbr_in(&p, all_qmodes, vec);
+    save_abbr_in(&p, all_bmodes, vec);
+    save_abbr_in(&p, all_wmodes, vec);
+    save_abbr_in(&p, all_mmodes, vec);
+    if (p) {
+	InsertSorted(&all_modes, vec[2], p->Data, "", p->Cond, "", "");
+    } else {
+	fprintf(stderr, "Did not find \"%s\"\n", vec[1]);
+	longjmp(my_top, 1);
+    }
+}
+
 /******************************************************************************/
 static void
 save_all_submodes(
@@ -1166,16 +1300,18 @@ save_all_submodes(
 {
     if (isboolean(*type)) {
 	char t_normal[LEN_BUFFER], t_abbrev[LEN_BUFFER];
-	save_all_submodes("Bool",
-			  strcat(strcpy(t_normal, "no"), normal),
-			  *abbrev
-			  ? strcat(strcpy(t_abbrev, "no"), abbrev)
-			  : "",
-			  cond);
+	my_strncat(strcpy(t_normal, "no"), normal, sizeof(t_normal));
+	if (*abbrev) {
+	    my_strncat(strcpy(t_abbrev, "no"), abbrev, sizeof(t_abbrev));
+	} else {
+	    strcpy(t_abbrev, "");
+	}
+	save_all_submodes("Bool", t_normal, t_abbrev, cond);
+    }
+    InsertSorted(&all_submodes, normal, type, "", cond, "", "");
+    if (*abbrev && strcmp(normal, abbrev)) {
+	InsertSorted(&all_submodes, abbrev, type, "", cond, "", "");
     }
-    InsertSorted(&all_submodes, normal, type, "", cond, "");
-    if (*abbrev)
-	InsertSorted(&all_submodes, abbrev, type, "", cond, "");
 }
 
 static void
@@ -1228,8 +1364,9 @@ predefine_submodes(char **vec, int len)
 		break;
 	    }
 	}
-	if (!found)
-	    InsertSorted(&all_majors, vec[2], "", "", "", "");
+	if (!found) {
+	    InsertSorted(&all_majors, vec[2], "", "", "", "", "");
+	}
 	if (len > 2) {
 	    for (p = all_bmodes, found = FALSE; p; p = p->nst) {
 		if (sscanf(p->Name, "%s\n%s\n%s",
@@ -1242,9 +1379,9 @@ predefine_submodes(char **vec, int len)
 	    }
 	    if (found) {
 		Sprintf(temp, "%s-%s", vec[2], norm);
-		strcpy(norm, temp);
+		my_strncpy(norm, temp, sizeof(norm));
 		Sprintf(temp, "%s%s", vec[2], abbr);
-		strcpy(abbr, temp);
+		my_strncpy(abbr, temp, sizeof(abbr));
 		save_all_modes(c2TYPE(*type), norm, abbr,
 			       formcond(p->Cond, "OPT_MAJORMODE"));
 	    }
@@ -1257,7 +1394,7 @@ static void
 save_qmodes(char *type, char **vec)
 {
     char *key = Mode2Key(type, vec[1], vec[4], TRUE);
-    InsertSorted(&all_qmodes, key, vec[2], vec[3], vec[4], vec[0]);
+    InsertSorted(&all_qmodes, key, vec[2], vec[3], vec[4], vec[0], "");
 #if NO_LEAKS
     free(key);
 #endif
@@ -1302,11 +1439,17 @@ dump_qmodes(void)
 }
 
 /******************************************************************************/
+static int
+ok_submode(char *side_effect)
+{
+    return strcmp(side_effect, "chgd_major") != 0;
+}
+
 static void
 save_bmodes(char *type, char **vec)
 {
-    char *key = Mode2Key(type, vec[1], vec[4], TRUE);
-    InsertSorted(&all_bmodes, key, vec[2], vec[3], vec[4], vec[0]);
+    char *key = Mode2Key(type, vec[1], vec[4], ok_submode(vec[3]));
+    InsertSorted(&all_bmodes, key, vec[2], vec[3], vec[4], vec[0], "");
 #if NO_LEAKS
     free(key);
 #endif
@@ -1356,6 +1499,8 @@ start_vars_h(char **argv)
 {
     static const char *const head[] =
     {
+	"",
+	"#include <blist.h>",
 	"",
 	"#if OPT_EVAL",
 	"",
@@ -1365,6 +1510,7 @@ start_vars_h(char **argv)
 	"\tstruct UVAR *next;",
 	"\tchar *u_name;\t\t/* name of temp variable */",
 	"\tchar *u_value;\t\t/* value (string) */",
+	"\tchar u_type;\t\t/* type (normally string) */",
 	"} UVAR;",
 	"",
 	"decl_uninit( UVAR *temp_vars );\t/* temporary variables */",
@@ -1393,6 +1539,8 @@ init_statevars(void)
 	"",
 	"/*\tlist of recognized state variables\t*/",
 	"",
+	"extern BLIST blist_statevars;",
+	"",
 	"#ifdef realdef",
 	"DECL_EXTERN_CONST(char *const statevars[]) = {"
     };
@@ -1402,15 +1550,24 @@ init_statevars(void)
 	write_lines(nevars, head);
 }
 
+#define NonNull(s) ((s) ? (s) : "")
+/*
+ * vec[0] is empty
+ * vec[1] is variable name
+ * vec[2] is symbol
+ * vec[3] is condition
+ * vec[4] is comment (unused so far)
+ */
 static void
-save_statevars(char **vec)
+save_statevars(char *type, char **vec)
 {
-    /* insert into 'all_modes' to provide for common name-completion
-     * table, and into 'all_statevars' to get name/index correspondence.
-     */
-    InsertSorted(&all_modes, vec[1], "state", "", formcond("OPT_EVAL",
-							   vec[3]), "");
-    InsertSorted(&all_statevars, vec[1], vec[2], "", vec[3], vec[0]);
+    char *name = vec[1];
+    char *vars = vec[2];
+    char *cond = vec[3];
+    char *key = Vars2Key(type, name, cond);
+
+    InsertSorted(&all_statevars, key, vars, "", cond, vec[4], "");
+    free(key);
 }
 
 static void
@@ -1420,6 +1577,7 @@ dump_statevars(void)
     {
 	"\tNULL\t/* ends table for name-completion */",
 	"};",
+	"BLIST blist_statevars = init_blist(statevars);",
 	"#else",
 	"extern const char *const statevars[];",
 	"#endif",
@@ -1439,29 +1597,50 @@ dump_statevars(void)
 	"",
 	"typedef int (StateFunc)(TBUFF **resultp, const char *valuep);",
 	"",
+	"typedef struct {",
+	"\tStateFunc *func;",
+	"\tchar type;",
+	"#if OPT_ONLINEHELP",
+	"\tconst char* help;",
+	"#endif",
+	"} StateVars;",
 	"",
 	"#ifdef realdef",
-	"DECL_EXTERN(StateFunc *statevar_func[]) = {",
+	"",
+	"#undef DATA",
+	"#if OPT_ONLINEHELP",
+	"#define DATA(func,type,help) { func, type, help }",
+	"#else",
+	"#define DATA(func,type,help) { func, type}",
+	"#endif",
+	"",
+	"DECL_EXTERN(const StateVars statevar_funcs[]) = {",
 	""
     };
     static const char *const tail[] =
     {
-	"\t(StateFunc *)NULL",
+	"\tDATA( (StateFunc *)NULL, 0, 0 )",
 	"};",
 	"#else",
-	"extern StateFunc *statevar_func[];",
+	"extern const StateVars statevar_funcs[];",
 	"#endif /* realdef */",
 	"",
     };
+    char temp[LEN_BUFFER], norm[LEN_BUFFER], type[LEN_BUFFER];
     LIST *p;
     char *s;
+    int first = 1;
 
     BeginIf();
     for (p = all_statevars; p != 0; p = p->nst) {
-	if (p == all_statevars)
+	if (sscanf(p->Name, "%s\n%s", norm, type) != 2)
+	    continue;
+	if (first) {
 	    init_statevars();
+	    first = 0;
+	}
 	WriteIf(nevars, p->Cond);
-	Fprintf(nevars, "\t%s,\n", s = Name2Symbol(p->Name));
+	Fprintf(nevars, "\t%s,\n", s = Name2Symbol(norm));
 	free(s);
     }
     FlushIf(nevars);
@@ -1484,8 +1663,15 @@ dump_statevars(void)
     write_lines(nevars, middle2);
     /* emit the variable get/set routine table */
     for (p = all_statevars; p != 0; p = p->nst) {
+	if (sscanf(p->Name, "%s\n%s", norm, type) != 2)
+	    continue;
 	WriteIf(nevars, p->Cond);
-	Fprintf(nevars, "\tvar_%s,\n", p->Func);
+	Sprintf(temp, "\tDATA( var_%s,", p->Func);
+	(void) PadTo(32, temp);
+	Sprintf(temp + strlen(temp), "VALTYPE_%s,", c2TYPE(*type));
+	(void) PadTo(50, temp);
+	Sprintf(temp + strlen(temp), "\"%s\" ),", p->Note);
+	Fprintf(nevars, "%s\n", temp);
     }
     FlushIf(nevars);
     write_lines(nevars, tail);
@@ -1495,28 +1681,36 @@ dump_statevars(void)
 static void
 save_fsms(char **vec)
 {
-    InsertSorted(&all_fsms, vec[1], "", vec[2], vec[3], vec[0]);
+    InsertSorted(&all_fsms,
+		 lowercase(vec[1]),
+		 "",
+		 vec[2],
+		 vec[3],
+		 vec[0],
+		 "");
 }
 
 static void
 init_fsms(void)
 {
-    char name[BUFSIZ];
     int n;
 
-    if (fsm_name == 0)
+    if (fsm_uc_name == 0) {
 	badfmt("Missing table name");
+    } else {
 
-    (void) strcpy(name, fsm_name);
-    for (n = 0; fsm_name[n] != '\0'; n++)
-	fsm_name[n] = (char) toUpper(fsm_name[n]);
-    Fprintf(nefsms, "\n");
-    Fprintf(nefsms, "#if OPT_%s_CHOICES\n", fsm_name);
-    Fprintf(nefsms, "#ifndef realdef\n");
-    Fprintf(nefsms, "extern const FSM_CHOICES fsm_%s_choices[];\n", name);
-    Fprintf(nefsms, "#else\n");
-    Fprintf(nefsms, "DECL_EXTERN_CONST(FSM_CHOICES fsm_%s_choices[]) = %c\n",
-	    name, L_CURL);
+	for (n = 0; fsm_uc_name[n] != '\0'; n++)
+	    fsm_uc_name[n] = (char) toUpper(fsm_uc_name[n]);
+	Fprintf(nefsms, "\n");
+	Fprintf(nefsms, "extern FSM_BLIST fsm_%s_blist;\n\n", fsm_lc_name);
+	Fprintf(nefsms, "#if OPT_%s_CHOICES\n", fsm_uc_name);
+	Fprintf(nefsms, "#ifndef realdef\n");
+	Fprintf(nefsms, "extern const FSM_CHOICES fsm_%s_choices[];\n", fsm_lc_name);
+	Fprintf(nefsms, "#else\n");
+	Fprintf(nefsms,
+		"DECL_EXTERN_CONST(FSM_CHOICES fsm_%s_choices[]) = %c\n",
+		fsm_lc_name, L_CURL);
+    }
 }
 
 static void
@@ -1543,13 +1737,22 @@ dump_fsms(void)
 	FlushIf(nefsms);
 
 	write_lines(nefsms, middle);
+
+	Fprintf(nefsms, "FSM_BLIST fsm_%s_blist = {\n", fsm_lc_name);
+	Fprintf(nefsms, "\tfsm_%s_choices,\n", fsm_lc_name);
+	Fprintf(nefsms, "\tinit_blist(fsm_%s_choices)\n", fsm_lc_name);
+	Fprintf(nefsms, "};\n");
+
 	Fprintf(nefsms, "#endif\n");
-	Fprintf(nefsms, "#endif /* OPT_%s_CHOICES */\n", fsm_name);
+	Fprintf(nefsms, "#endif /* OPT_%s_CHOICES */\n", fsm_lc_name);
 
 	free_LIST(&all_fsms);
 
-	free(fsm_name);
-	fsm_name = 0;
+	free(fsm_uc_name);
+	fsm_uc_name = 0;
+
+	free(fsm_lc_name);
+	fsm_lc_name = 0;
     }
 }
 
@@ -1568,7 +1771,8 @@ start_fsms_h(char **argv, char *name)
 	write_lines(nefsms, head);
     }
     dump_fsms();
-    fsm_name = StrAlloc(name);
+    fsm_uc_name = StrAlloc(name);
+    fsm_lc_name = StrAlloc(name);
 }
 
 static void
@@ -1605,6 +1809,7 @@ save_funcs(
 {
     char temp[MAX_BUFFER];
     char *s;
+    LIST *p;
 
     if (strcmp(cond, old_cond)) {
 	if (*old_cond) {
@@ -1623,27 +1828,51 @@ save_funcs(
     Sprintf(temp, "extern int %s ( int f, int n );", func);
     InsertOnEnd(&all_funcs, temp);
 
-    s = strcpy(temp, "\t");
+    s = strcpy(temp, "\n");
+
+    s = append(s, "static const char *syn_");
+    s = append(s, func);
+    s = append(s, "[] = \n{\n");
+    for (p = all_aliases; p != 0; p = p->nst) {
+	if (p->Cond[0]) {
+	    s = append(s, "#if ");
+	    s = append(s, p->Cond);
+	    s = append(s, "\n");
+	}
+	s = append(s, "\t\"");
+	s = append(s, p->Name);
+	s = append(s, "\",\n");
+	if (p->Cond[0]) {
+	    s = append(s, "#endif\n");
+	}
+    }
+    s = append(s, "\t0\n};\n");
+    s = append(s, "\n");
+
     s = append(s, "DECL_EXTERN_CONST(CMDFUNC f_");
     s = append(s, func);
     s = append(s, ")");
     (void) PadTo(32, temp);
-    s = append(s, "= {\n\tINIT_UNION(");
+    s = append(s, "=\n{\n\tINIT_UNION(");
     s = append(s, func);
     s = append(s, "),\n\t");
     s = append(s, flags);
-    s = append(s, "\n#if OPT_MACRO_ARGS\n\t\t,0\n#endif");
-    s = append(s, "\n#if OPT_TRACE\n\t\t,\"");
+    s = append(s, "\n\t,syn_");
+    s = append(s, func);
+    s = append(s, "\n#if OPT_MACRO_ARGS\n\t,0\n#endif");
+    s = append(s, "\n#if OPT_TRACE\n\t,\"");
     s = append(s, func);
     s = append(s, "\"\n#endif");
-    s = append(s, "\n#if OPT_ONLINEHELP\n\t\t,\"");
+    s = append(s, "\n#if OPT_ONLINEHELP\n\t,\"");
     s = append(s, help);
-    (void) append(s, "\"\n#endif\n };");
+    (void) append(s, "\"\n#endif\n};");
     InsertOnEnd(&all__FUNCs, temp);
 
     s = append(strcpy(temp, "extern const CMDFUNC f_"), func);
     (void) append(s, ";");
     InsertOnEnd(&all__CMDFs, temp);
+
+    free_LIST(&all_aliases);
 }
 
 static void
@@ -1660,7 +1889,7 @@ save_gmodes(char *type, char **vec)
 {
     char *key = Mode2Key(type, vec[1], vec[4], FALSE);
 
-    InsertSorted(&all_gmodes, key, vec[2], vec[3], vec[4], vec[0]);
+    InsertSorted(&all_gmodes, key, vec[2], vec[3], vec[4], vec[0], "");
 #if NO_LEAKS
     free(key);
 #endif
@@ -1706,9 +1935,15 @@ dump_gmodes(void)
 
 /******************************************************************************/
 static void
-save_names(char *name, char *func, char *cond)
+save_names(char *name, char *func, char *cond, char *flags)
 {
-    InsertSorted(&all_names, name, func, "", cond, "");
+    InsertSorted(&all_names, name, func, "", cond, "", flags);
+}
+
+static void
+save_aliases(char *name, char *func, char *cond, char *flags)
+{
+    InsertSorted(&all_aliases, name, func, "", cond, "", flags);
 }
 
 static void
@@ -1717,8 +1952,13 @@ dump_names(void)
     LIST *m;
     char temp[MAX_BUFFER];
 
-    Fprintf(nename, "\n/* if you maintain this by hand, keep it in */\n");
-    Fprintf(nename, "/* alphabetical order!!!! */\n\n");
+    Fprintf(nename, "#include <blist.h>\n");
+    Fprintf(nename, "\n");
+    Fprintf(nename, "extern BLIST blist_nametbl;\n");
+    Fprintf(nename, "extern BLIST blist_glbstbl;\n");
+    Fprintf(nename, "\n");
+    Fprintf(nename, "#ifdef real_NAMETBL\n");
+    Fprintf(nename, "\n");
     Fprintf(nename, "EXTERN_CONST NTAB nametbl[] = {\n");
 
     BeginIf();
@@ -1729,6 +1969,26 @@ dump_names(void)
     }
     FlushIf(nename);
     Fprintf(nename, "\t{ NULL, NULL }\n};\n");
+    Fprintf(nename, "\n");
+    Fprintf(nename, "BLIST blist_nametbl = init_blist(nametbl);\n");
+    Fprintf(nename, "\n");
+    Fprintf(nename, "EXTERN_CONST NTAB glbstbl[] = {\n");
+    BeginIf();
+    for (m = all_names; m != NULL; m = m->nst) {
+	if (m->Flag != 0 && strstr(m->Flag, "GLOBOK") != 0) {
+	    WriteIf(nename, m->Cond);
+	    Sprintf(temp, "\t{ \"%s\",", m->Name);
+	    Fprintf(nename, "%s&f_%s },\n", PadTo(40, temp), m->Func);
+	}
+    }
+    FlushIf(nename);
+    Fprintf(nename, "\t{ NULL, NULL }\n};\n");
+    Fprintf(nename, "\n");
+    Fprintf(nename, "BLIST blist_glbstbl = init_blist(glbstbl);\n");
+    Fprintf(nename, "\n");
+    Fprintf(nename, "#else\n");
+    Fprintf(nename, "\n");
+    Fprintf(nename, "#endif\n");
 }
 
 /******************************************************************************/
@@ -1749,6 +2009,9 @@ init_ufuncs(void)
 	"typedef struct UFUNC {",
 	"\tconst char *f_name;\t/* name of function */",
 	"\tunsigned f_code;",
+	"#if OPT_ONLINEHELP",
+	"\tconst char *f_help;",
+	"#endif",
 	"} UFUNC;",
 	"",
 	"#define NARGMASK	0x000f",
@@ -1760,6 +2023,14 @@ init_ufuncs(void)
 	"#define SRET		0x0400",
 	"",
 	"#ifdef realdef",
+	"",
+	"#undef DATA",
+	"#if OPT_ONLINEHELP",
+	"#define DATA(name,code,help) {name,(unsigned)(code),help}",
+	"#else",
+	"#define DATA(name,code,help) {name,(unsigned)(code)}",
+	"#endif",
+	"",
 	"DECL_EXTERN_CONST(UFUNC vl_ufuncs[]) = {",
     };
     static int done;
@@ -1782,15 +2053,23 @@ init_ufuncs(void)
 static void
 save_ufuncs(char **vec)
 {
-    InsertSorted(&all_ufuncs, vec[1], vec[2], vec[3], "", vec[0]);
+    InsertSorted(&all_ufuncs, vec[1], vec[2], vec[3], "", vec[4], "");
 }
 
 static void
 dump_ufuncs(void)
 {
+    static const char *const head[] =
+    {
+	"extern BLIST blist_ufuncs;",
+	"",
+    };
     static const char *const middle[] =
     {
+	"\tDATA(NULL, 0, \"\"),",
 	"};",
+	"#undef DATA",
+	"BLIST blist_ufuncs = init_blist(vl_ufuncs);",
 	"#else",
 	"extern const UFUNC vl_ufuncs[];",
 	"#endif",
@@ -1800,16 +2079,15 @@ dump_ufuncs(void)
     LIST *p;
     int count;
 
+    write_lines(nevars, head);
     for (p = all_ufuncs, count = 0; p != 0; p = p->nst) {
 	if (!count++)
 	    init_ufuncs();
-	Sprintf(temp, "\t{\"%s\",", p->Name);
-	(void) PadTo(15, temp);
-	Sprintf(temp + strlen(temp), "(unsigned)(%s)},", p->Data);
-	if (p->Note[0]) {
-	    (void) PadTo(32, temp);
-	    Sprintf(temp + strlen(temp), "/* %s */", p->Note);
-	}
+	Sprintf(temp, "\tDATA(\"%s\",", p->Name);
+	(void) PadTo(32, temp);
+	Sprintf(temp + strlen(temp), "%s,", p->Data);
+	(void) PadTo(48, temp);
+	Sprintf(temp + strlen(temp), "\"%s\"),", p->Note);
 	Fprintf(nevars, "%s\n", temp);
     }
     write_lines(nevars, middle);
@@ -1820,7 +2098,7 @@ static void
 save_wmodes(char *type, char **vec)
 {
     char *key = Mode2Key(type, vec[1], vec[4], FALSE);
-    InsertSorted(&all_wmodes, key, vec[2], vec[3], vec[4], vec[0]);
+    InsertSorted(&all_wmodes, key, vec[2], vec[3], vec[4], vec[0], "");
 #if NO_LEAKS
     free(key);
 #endif
@@ -1935,6 +2213,7 @@ mkw32binding(char *key, char *conditional, char *func, char *fcond)
 		 func,
 		 "",
 		 formcond(fcond, conditional),
+		 "",
 		 "");
 #undef KEYTOKEN
 }
@@ -1955,6 +2234,7 @@ free_mktbls(void)
     int k;
 
     free_LIST(&all_names);
+    free_LIST(&all_aliases);
     free_LIST(&all_funcs);
     free_LIST(&all__FUNCs);
     free_LIST(&all__CMDFs);
@@ -2047,6 +2327,9 @@ main(int argc, char *argv[])
 
 	case '.':		/* a new section */
 	    switch (col1) {
+	    case 'a':
+		section = SECT_ABBR;
+		break;
 	    case 'c':
 		section = SECT_CMDS;
 		break;
@@ -2085,13 +2368,19 @@ main(int argc, char *argv[])
 
 	case '\t':		/* a new function */
 	    switch (section) {
+	    case SECT_ABBR:
+		if (r != 2)
+		    badfmt("looking for ABBR assignments");
+		save_abbrs(vec);
+		break;
 	    case SECT_CMDS:
 		switch (col1) {
 		case '"':	/* then it's an english name */
 		    if (r < 1 || r > 2)
 			badfmt("looking for english name");
 
-		    save_names(vec[1], func, formcond(fcond, vec[2]));
+		    save_names(vec[1], func, formcond(fcond, vec[2]), flags);
+		    save_aliases(vec[1], func, formcond(fcond, vec[2]), flags);
 		    break;
 
 		case '\'':	/* then it's a key */
@@ -2101,8 +2390,8 @@ main(int argc, char *argv[])
 		    if (strcmp("W32KY", vec[2]) == 0) {
 			mkw32binding(vec[1], vec[3], func, fcond);
 		    } else {
-			if (strncmp("KEY_", vec[2], 4) == 0) {
-			    if (strncmp("FN-", vec[1], 3) != 0)
+			if (strncmp("KEY_", vec[2], (size_t) 4) == 0) {
+			    if (strncmp("FN-", vec[1], (size_t) 3) != 0)
 				badfmt("KEY_xxx definition must for FN- binding");
 			    if (!nefkeys)
 				nefkeys = OpenHeader("nefkeys.h", argv);
@@ -2116,7 +2405,7 @@ main(int argc, char *argv[])
 
 		case '<':	/* then it's a help string */
 		    /* put code here. */
-		    (void) strcpy(funchelp, vec[1]);
+		    (void) my_strncpy(funchelp, vec[1], sizeof(funchelp));
 		    break;
 
 		default:
@@ -2154,6 +2443,12 @@ main(int argc, char *argv[])
 		save_wmodes(modetype, vec);
 		break;
 
+	    case SECT_VARS:
+		if (r < 2 || r > 4)
+		    badfmt("looking for state variables");
+		save_statevars(modetype, vec);
+		break;
+
 	    default:
 		badfmt("did not expect a tab");
 	    }
@@ -2169,13 +2464,12 @@ main(int argc, char *argv[])
 		   a help line for it.  save the previous
 		   one now, and hang onto this one */
 		if (func[0]) {	/* flush the old one */
-		    save_funcs(func, flags, fcond,
-			       old_fcond, funchelp);
+		    save_funcs(func, flags, fcond, old_fcond, funchelp);
 		    funchelp[0] = EOS;
 		}
-		(void) strcpy(func, vec[1]);
-		(void) strcpy(flags, vec[2]);
-		(void) strcpy(fcond, vec[3]);
+		(void) my_strncpy(func, vec[1], sizeof(func));
+		(void) my_strncpy(flags, vec[2], sizeof(flags));
+		(void) my_strncpy(fcond, vec[3], sizeof(fcond));
 		break;
 
 	    case SECT_FSMS:
@@ -2183,13 +2477,18 @@ main(int argc, char *argv[])
 		break;
 
 	    case SECT_VARS:
-		if (r < 2 || r > 3)
-		    badfmt("looking for char *statevars[]");
-		save_statevars(vec);
+		if (r != 1
+		    || (!strcmp(vec[1], "bool")
+			&& !strcmp(vec[1], "enum")
+			&& !strcmp(vec[1], "int")
+			&& !strcmp(vec[1], "string")
+			&& !strcmp(vec[1], "regex")))
+		    badfmt("looking for mode datatype");
+		(void) my_strncpy(modetype, vec[1], sizeof(modetype));
 		break;
 
 	    case SECT_FUNC:
-		if (r < 2 || r > 3)
+		if (r < 2 || r > 4)
 		    badfmt("looking for UFUNC func[]");
 		save_ufuncs(vec);
 		break;
@@ -2206,7 +2505,7 @@ main(int argc, char *argv[])
 			&& !strcmp(vec[1], "string")
 			&& !strcmp(vec[1], "regex")))
 		    badfmt("looking for mode datatype");
-		(void) strcpy(modetype, vec[1]);
+		(void) my_strncpy(modetype, vec[1], sizeof(modetype));
 		break;
 
 	    default:
@@ -2214,6 +2513,7 @@ main(int argc, char *argv[])
 	    }
 	}
     }
+    fclose(cmdtbl);
 
     if (func[0]) {		/* flush the old one */
 	save_funcs(func, flags, fcond, old_fcond, funchelp);
@@ -2254,7 +2554,6 @@ main(int argc, char *argv[])
 
     if (all_wmodes || all_bmodes) {
 	nemode = OpenHeader("nemode.h", argv);
-	DefineOffset(nemode);
 	dump_majors();
 	dump_all_modes();
 	dump_all_submodes();
@@ -2265,6 +2564,23 @@ main(int argc, char *argv[])
 	dump_wmodes();
     }
 
+    if (nebind != 0)
+	fclose(nebind);
+    if (nefkeys != 0)
+	fclose(nefkeys);
+    if (nefsms != 0)
+	fclose(nefsms);
+    if (nefunc != 0)
+	fclose(nefunc);
+    if (nemode != 0)
+	fclose(nemode);
+    if (nename != 0)
+	fclose(nename);
+    if (neprot != 0)
+	fclose(neprot);
+    if (nevars != 0)
+	fclose(nevars);
+
     free_mktbls();
     return (GOODEXIT);
 }
