@@ -3,7 +3,7 @@
  * and backward directions.
  *  heavily modified by Paul Fox, 1990
  *
- * $Header: /usr/build/vile/vile/RCS/search.c,v 1.137 2005/01/26 23:04:42 tom Exp $
+ * $Header: /usr/build/vile/vile/RCS/search.c,v 1.154 2015/01/19 10:17:47 tom Exp $
  *
  * original written Aug. 1986 by John M. Gamble, but I (pgf) have since
  * replaced his regex stuff with Henry Spencer's regexp package.
@@ -49,7 +49,7 @@ scrbacksearch(int f, int n)
 int
 forwsearch(int f, int n)
 {
-    register int status;
+    int status;
     hst_init('/');
     status = fsearch(f, n, FALSE, FALSE);
     hst_flush();
@@ -62,19 +62,21 @@ forwsearch(int f, int n)
 int
 fsearch(int f, int n, int marking, int fromscreen)
 {
-    register int status = TRUE;
+    int status = TRUE;
     int wrapok;
     MARK curpos;
     int didmark = FALSE;
     int didwrap;
 
+    assert(curwp != 0);
+
     if (f && n < 0)
 	return rsearch(f, -n, FALSE, FALSE);
 
     if (n == 0)
 	n = 1;
 
-    wrapok = marking || window_b_val(curwp, MDSWRAP);
+    wrapok = marking || window_b_val(curwp, MDWRAPSCAN);
 
     last_srch_direc = FORWARD;
 
@@ -92,12 +94,15 @@ fsearch(int f, int n, int marking, int fromscreen)
 
     ignorecase = window_b_val(curwp, MDIGNCASE);
 
+    if (curwp == 0)
+	return FALSE;
+
     curpos = DOT;
     scanboundry(wrapok, curpos, FORWARD);
     didwrap = FALSE;
     while (marking || n--) {
 	movenext(&(DOT), FORWARD);
-	status = scanner(gregexp, FORWARD, wrapok, &didwrap);
+	status = scanner(gregexp, FORWARD, wrapok, TRUE, &didwrap);
 	if (status == ABORT) {
 	    mlwarn("[Aborted]");
 	    DOT = curpos;
@@ -153,12 +158,14 @@ fsearch(int f, int n, int marking, int fromscreen)
 int
 forwhunt(int f, int n)
 {
-    register int status = TRUE;
+    int status = TRUE;
     int wrapok;
     MARK curpos;
     int didwrap;
 
-    wrapok = window_b_val(curwp, MDSWRAP);
+    assert(curwp != 0);
+
+    wrapok = window_b_val(curwp, MDWRAPSCAN);
 
     if (f && n < 0)		/* search backwards */
 	return (backhunt(f, -n));
@@ -174,6 +181,9 @@ forwhunt(int f, int n)
 
     ignorecase = window_b_val(curwp, MDIGNCASE);
 
+    if (curwp == 0)
+	return FALSE;
+
     /* find n'th occurrence of pattern
      */
     curpos = DOT;
@@ -181,7 +191,7 @@ forwhunt(int f, int n)
     didwrap = FALSE;
     while (n--) {
 	movenext(&(DOT), FORWARD);
-	status = scanner(gregexp, FORWARD, wrapok, &didwrap);
+	status = scanner(gregexp, FORWARD, wrapok, TRUE, &didwrap);
 	if (didwrap) {
 	    mlwrite("[Search wrapped past end of buffer]");
 	    didwrap = FALSE;
@@ -217,7 +227,7 @@ forwhunt(int f, int n)
 int
 backsearch(int f, int n)
 {
-    register int status;
+    int status;
     hst_init('?');
     status = rsearch(f, n, FALSE, FALSE);
     hst_flush();
@@ -228,18 +238,20 @@ backsearch(int f, int n)
 static int
 rsearch(int f, int n, int dummy GCC_UNUSED, int fromscreen)
 {
-    register int status;
+    int status;
     int wrapok;
     MARK curpos;
     int didwrap;
 
+    assert(curwp != 0);
+
     if (f && n < 0)		/* reverse direction */
 	return fsearch(f, -n, FALSE, fromscreen);
 
     if (n == 0)
 	n = 1;
 
-    wrapok = window_b_val(curwp, MDSWRAP);
+    wrapok = window_b_val(curwp, MDWRAPSCAN);
 
     last_srch_direc = REVERSE;
 
@@ -253,12 +265,15 @@ rsearch(int f, int n, int dummy GCC_UNUSED, int fromscreen)
 
     ignorecase = window_b_val(curwp, MDIGNCASE);
 
+    if (curwp == 0)
+	return FALSE;
+
     curpos = DOT;
     scanboundry(wrapok, DOT, REVERSE);
     didwrap = FALSE;
     while (n--) {
 	movenext(&(DOT), REVERSE);
-	status = scanner(gregexp, REVERSE, wrapok, &didwrap);
+	status = scanner(gregexp, REVERSE, wrapok, TRUE, &didwrap);
 	if (didwrap) {
 	    mlwrite(
 		       "[Search wrapped past start of buffer]");
@@ -291,12 +306,14 @@ rsearch(int f, int n, int dummy GCC_UNUSED, int fromscreen)
 int
 backhunt(int f, int n)
 {
-    register int status = TRUE;
+    int status = TRUE;
     int wrapok;
     MARK curpos;
     int didwrap;
 
-    wrapok = window_b_val(curwp, MDSWRAP);
+    assert(curwp != 0);
+
+    wrapok = window_b_val(curwp, MDWRAPSCAN);
 
     if (f && n < 0)		/* search forwards */
 	return (forwhunt(f, -n));
@@ -312,6 +329,9 @@ backhunt(int f, int n)
 
     ignorecase = window_b_val(curwp, MDIGNCASE);
 
+    if (curwp == 0)
+	return FALSE;
+
     /* find n'th occurrence of pattern
      */
     curpos = DOT;
@@ -319,7 +339,7 @@ backhunt(int f, int n)
     didwrap = FALSE;
     while (n--) {
 	movenext(&(DOT), REVERSE);
-	status = scanner(gregexp, REVERSE, wrapok, &didwrap);
+	status = scanner(gregexp, REVERSE, wrapok, TRUE, &didwrap);
 	if (didwrap) {
 	    mlwrite("[Search wrapped past start of buffer]");
 	    didwrap = FALSE;
@@ -369,7 +389,7 @@ revsearch(int f, int n)
 static int
 testit(LINE *lp, regexp * exp, int *end, int srchlim)
 {
-    char *txt = lp->l_text;
+    char *txt = lvalue(lp);
     C_NUM col = (C_NUM) (exp->startp[0] - txt) + 1;
 
     if (col > llength(lp))
@@ -395,16 +415,22 @@ scanner(
 	   regexp * exp,	/* the compiled expression */
 	   int direct,		/* up or down */
 	   int wrapok,		/* ok to wrap around end of buffer? */
-	   int *wrappedp)
+	   int at_bol,		/* ok to match "^" ? */
+	   int *wrappedp)	/* in/out: tells if we wrapped around */
 {
     MARK curpos;
     int found;
     int wrapped = FALSE;
     int leftmargin = b_left_margin(curbp);
 
+    TRACE((T_CALLED "scanner %s%s %s\n",
+	   (at_bol ? "bol " : ""),
+	   ((direct == FORWARD) ? "forward" : "backward"),
+	   (wrapok ? "wrapok" : "nowrapok")));
+
     if (!exp) {
 	mlforce("BUG: null exp");
-	return FALSE;
+	returnCode(FALSE);
     }
 
     /* Set starting search position to current position
@@ -415,12 +441,12 @@ scanner(
 
     /* Scan each character until we hit the scan boundary */
     for_ever {
-	register int startoff, srchlim;
+	int startoff, srchlim;
 
 	if (interrupted()) {
 	    if (wrappedp)
 		*wrappedp = wrapped;
-	    return ABORT;
+	    returnCode(ABORT);
 	}
 
 	if (sameline(curpos, scanboundpos)) {
@@ -448,7 +474,7 @@ scanner(
 			srchlim = scanboundpos.o + 1;
 		    }
 		}
-		found = lregexec(exp, curpos.l, startoff, srchlim);
+		found = cregexec(exp, curpos.l, startoff, srchlim, at_bol);
 	    }
 	} else {
 	    if (direct == FORWARD) {
@@ -460,10 +486,10 @@ scanner(
 		if (srchlim > llength(curpos.l))
 		    srchlim = llength(curpos.l);
 	    }
-	    found = lregexec(exp, curpos.l, startoff, srchlim);
+	    found = cregexec(exp, curpos.l, startoff, srchlim, at_bol);
 	}
 	if (found) {
-	    char *txt = curpos.l->l_text;
+	    char *txt = lvalue(curpos.l);
 	    char *got = exp->startp[0];
 	    C_NUM next;
 	    C_NUM last = curpos.o;
@@ -484,13 +510,14 @@ scanner(
 		}
 		if (end)
 		    last++;
-		if (!lregexec(exp, curpos.l, (int) (got - txt), srchlim)) {
+		if (!cregexec(exp, curpos.l, (int) (got - txt), srchlim, at_bol)) {
 		    mlforce("BUG: prev. match no good");
-		    return FALSE;
+		    returnCode(FALSE);
 		}
 	    } else if (llength(curpos.l) <= leftmargin
-		       || last < llength(curpos.l))
+		       || last < llength(curpos.l)) {
 		last--;
+	    }
 	    next = (C_NUM) (got - txt);
 	    if (next != last) {
 		DOT.l = curpos.l;
@@ -498,7 +525,10 @@ scanner(
 		curwp->w_flag |= WFMOVE;	/* flag that we have moved */
 		if (wrappedp)
 		    *wrappedp = wrapped;
-		return TRUE;
+#if OPT_TRACE
+		trace_mark("...scanner", &DOT, curbp);
+#endif
+		returnCode(TRUE);
 	    }
 	} else {
 	    if (sameline(curpos, scanboundpos) &&
@@ -526,12 +556,13 @@ scanner(
 	    if ((curpos.o = llength(curpos.l) - 1) < leftmargin)
 		curpos.o = leftmargin;
 	}
+	at_bol = TRUE;
 
     }
 
     if (wrappedp)
 	*wrappedp = wrapped;
-    return FALSE;		/* We could not find a match. */
+    returnCode(FALSE);		/* We could not find a match. */
 }
 
 #if OPT_HILITEMATCH
@@ -620,6 +651,8 @@ attrib_matches(void)
     REGIONSHAPE oregionshape = regionshape;
     VIDEO_ATTR vattr;
 
+    assert(curwp != 0);
+
     ignorecase = window_b_val(curwp, MDIGNCASE);
 
     if (!need_to_rehilite())
@@ -639,6 +672,9 @@ attrib_matches(void)
 
     (void) clear_match_attrs(TRUE, 1);
 
+    if (curwp == 0)
+	return;
+
     origdot = DOT;
     DOT.l = buf_head(curbp);
     DOT.o = 0;
@@ -652,7 +688,7 @@ attrib_matches(void)
 	    movenext(&nextdot, FORWARD);
 	    DOT = nextdot;
 	}
-	status = scanner(gregexp, FORWARD, FALSE, (int *) 0);
+	status = scanner(gregexp, FORWARD, FALSE, TRUE, (int *) 0);
 	if (status != TRUE)
 	    break;
 	if (vattr != VACOLOR)
@@ -669,15 +705,15 @@ attrib_matches(void)
 		videoattribute = VCOLORATTR(c + 1);
 	}
 	MK.l = DOT.l;
-	MK.o = DOT.o + gregexp->mlen;
+	MK.o = DOT.o + (C_NUM) gregexp->mlen;
 
 	/* provide a location for the next non-overlapping match */
 	nextdot = MK;
 	if (gregexp->mlen > 0)
-	    nextdot.o -= 1;
+	    nextdot.o -= BytesBefore(nextdot.l, nextdot.o);
 
 	/* show highlighting from DOT to MK */
-	regionshape = EXACT;
+	regionshape = rgn_EXACT;
 	videoattribute |= VOWN_MATCHES;
 	status = attributeregion();
     } while (status == TRUE);
@@ -728,12 +764,12 @@ readpattern(const char *prompt,
     TRACE((T_CALLED "readpattern(%s, %s, %p, %d, %d)\n",
 	   prompt ? prompt : "",
 	   tb_visible(*apat),
-	   srchexpp,
+	   (void *) srchexpp,
 	   c,
 	   fromscreen));
 
     if (fromscreen) {
-	if ((status = screen_string(temp, sizeof(temp), vl_ident)) == TRUE) {
+	if ((status = screen_to_ident(temp, sizeof(temp))) == TRUE) {
 	    if (tb_init(apat, EOS) == 0
 		|| tb_bappend(apat, temp, strlen(temp)) == 0) {
 		status = FALSE;
@@ -786,7 +822,7 @@ static void
 savematch(MARK curpos, size_t matchlen)
 {
     tb_init(&tb_matched_pat, EOS);
-    tb_bappend(&tb_matched_pat, &(curpos.l->l_text[curpos.o]), matchlen);
+    tb_bappend(&tb_matched_pat, &(lvalue(curpos.l)[curpos.o]), matchlen);
     tb_append(&tb_matched_pat, EOS);
 }
 
@@ -810,8 +846,8 @@ scanboundry(int wrapok, MARK dot, int dir)
 static void
 movenext(MARK *pdot, int dir)
 {
-    register LINE *curline;
-    register int curoff;
+    LINE *curline;
+    int curoff;
 
     curline = pdot->l;
     curoff = pdot->o;
@@ -820,14 +856,14 @@ movenext(MARK *pdot, int dir)
 	    curline = lforw(curline);	/* skip to next line */
 	    curoff = 0;
 	} else {
-	    curoff++;
+	    curoff += BytesAt(curline, curoff);
 	}
     } else {
 	if (curoff == 0) {
 	    curline = lback(curline);
 	    curoff = llength(curline);
 	} else {
-	    curoff--;
+	    curoff -= BytesBefore(curline, curoff);
 	}
     }
     pdot->l = curline;
@@ -845,8 +881,7 @@ findpat(int f, int n, regexp * exp, int direc)
     if (!exp)
 	return FALSE;
 
-    if (!f)
-	n = 1;
+    n = need_a_count(f, n, 1);
 
     s = TRUE;
     scanboundpos = curbp->b_line;	/* was scanboundry(FALSE,savepos,0); */
@@ -858,7 +893,7 @@ findpat(int f, int n, regexp * exp, int direc)
 	     ? forwchar(TRUE, 1)
 	     : backchar(TRUE, 1));
 	if (s == TRUE)
-	    s = scanner(exp, direc, FALSE, (int *) 0);
+	    s = scanner(exp, direc, FALSE, TRUE, (int *) 0);
     }
     if (s != TRUE)
 	DOT = savepos;
