@@ -8,7 +8,7 @@
 /************************************************************************/
 
 /*
- * $Header: /usr/build/vile/vile/RCS/x11menu.c,v 1.8 2005/01/18 01:08:06 tom Exp $
+ * $Header: /usr/build/vile/vile/RCS/x11menu.c,v 1.18 2013/12/07 16:26:12 tom Exp $
  */
 
 #define NEED_X_INCLUDES 1
@@ -54,7 +54,7 @@ static MY_MENUS *my_menus;
 /* Motif function to make a cascade button into a menubar               */
 /************************************************************************/
 void *
-gui_make_menu(void *menubar, char *nom, int the_class GCC_UNUSED)
+gui_make_menu(void *menubar, const char *nom, int the_class GCC_UNUSED)
 {
     MY_MENUS *remember;
     Widget pm;
@@ -94,10 +94,9 @@ gui_make_menu(void *menubar, char *nom, int the_class GCC_UNUSED)
 		      Nval(XmNmenuHelpWidget, cascade),
 		      NULL);
     }
-#endif /* MOTIF_WIDGETS */
-#if ATHENA_WIDGETS
+#elif ATHENA_WIDGETS
     static Widget last;
-    String str = XtMalloc(strlen(nom) + 10);
+    char *str = XtMalloc((Cardinal) strlen(nom) + 10);
 
     sprintf(str, "%sMenu", nom);
     pm = XtVaCreatePopupShell(str,
@@ -114,6 +113,20 @@ gui_make_menu(void *menubar, char *nom, int the_class GCC_UNUSED)
 				      Nval(XtNfromHoriz, last),
 				      Nval(XtNmenuName, str),
 				      NULL);
+#if OPT_MENUS_COLORED
+    XtVaSetValues(pm,
+		  Nval(XtNforeground, x_menu_foreground()),
+		  Nval(XtNbackground, x_menu_background()),
+		  NULL);
+    XtVaSetValues(menub,
+		  Nval(XtNforeground, x_menu_foreground()),
+		  Nval(XtNbackground, x_menu_background()),
+		  NULL);
+    XtVaSetValues(cascade,
+		  Nval(XtNforeground, x_menu_foreground()),
+		  Nval(XtNbackground, x_menu_background()),
+		  NULL);
+#endif /* OPT_MENUS_COLORED */
     last = cascade;
 
 #endif /* ATHENA_WIDGETS */
@@ -133,7 +146,7 @@ gui_make_menu(void *menubar, char *nom, int the_class GCC_UNUSED)
 /* Motif function to make a button into a cascade (with accelarator)    */
 /************************************************************************/
 void *
-gui_add_menu_item(void *pm, char *nom, char *accel GCC_UNUSED, int the_class)
+gui_add_menu_item(void *pm, const char *nom, char *accel GCC_UNUSED, int the_class)
 {
     Widget w;
 #if MOTIF_WIDGETS
@@ -168,9 +181,7 @@ gui_add_menu_item(void *pm, char *nom, char *accel GCC_UNUSED, int the_class)
     XmStringFree(xms_accl);
     XmStringFree(xms_name);
 
-    return w;
-#endif /* MOTIF_WIDGETS */
-#if ATHENA_WIDGETS
+#elif ATHENA_WIDGETS
     w = XtVaCreateManagedWidget("menuEntry",
 				((the_class == 'B')
 				 ? smeBSBObjectClass
@@ -188,7 +199,9 @@ gui_add_menu_item(void *pm, char *nom, char *accel GCC_UNUSED, int the_class)
 /* A buffer button takes part from the list tokens (L)                  */
 /************************************************************************/
 static void
-list_proc_back(Widget w GCC_UNUSED, XtPointer bname, XtPointer call GCC_UNUSED)
+list_proc_back(Widget w GCC_UNUSED,
+	       XtPointer bname GCC_UNUSED,
+	       XtPointer call GCC_UNUSED)
 {
     int num_buff;
     int oldflag = im_waiting(-1);
@@ -202,9 +215,8 @@ list_proc_back(Widget w GCC_UNUSED, XtPointer bname, XtPointer call GCC_UNUSED)
 #endif
     XmStringGetLtoR(xms, XmFONTLIST_DEFAULT_TAG, &accel);
     num_buff = atoi(&accel[1]);
-#endif
-#if ATHENA_WIDGETS
-    num_buff = (int) bname;
+#elif ATHENA_WIDGETS
+    num_buff = (int) (long) bname;
 #endif
 
     if (vile_is_busy)
@@ -258,8 +270,7 @@ gui_add_func_callback(void *w, void *closure)
 {
 #if MOTIF_WIDGETS
     XtAddCallback((Widget) w, XmNactivateCallback, proc_back, closure);
-#endif
-#if ATHENA_WIDGETS
+#elif ATHENA_WIDGETS
     XtAddCallback((Widget) w, XtNcallback, proc_back, closure);
 #endif /* ATHENA_WIDGETS */
 }
@@ -273,13 +284,13 @@ static void
 post_buffer_list(Widget w GCC_UNUSED,
 		 XtPointer client GCC_UNUSED,
 		 XEvent * ev GCC_UNUSED,
-		 Boolean * ok GCC_UNUSED)
+		 Boolean *ok GCC_UNUSED)
 {
-    static int in_item_menu_list = 0;	/* number allocated */
-    static int nb_item_menu_list = 0;	/* number in use */
+    static unsigned in_item_menu_list = 0;	/* number allocated */
+    static unsigned nb_item_menu_list = 0;	/* number in use */
     static Widget *pm_buffer;
 
-    int i, n = nb_item_menu_list;
+    unsigned i, n = nb_item_menu_list;
     BUFFER *bp;
     char string[NBUFN + 2 + NFILEN], temp[1 + NFILEN], *p;
     Widget pm = (Widget) client;
@@ -291,7 +302,7 @@ post_buffer_list(Widget w GCC_UNUSED,
 	if (b_is_temporary(bp))	/* cf: hist_show() */
 	    continue;
 
-	p = shorten_path(strcpy(temp, bp->b_fname), FALSE);
+	p = shorten_path(vl_strncpy(temp, bp->b_fname, sizeof(temp)), FALSE);
 	sprintf(string, "%-*.*s %s", NBUFN - 1, NBUFN - 1,
 		bp->b_bname, p);
 
@@ -299,7 +310,7 @@ post_buffer_list(Widget w GCC_UNUSED,
 	TRACE(("ACCEL(%s) = %s\n", temp, string));
 
 	if (nb_item_menu_list + 2 >= in_item_menu_list) {
-	    int m = in_item_menu_list;
+	    unsigned m = in_item_menu_list;
 
 	    in_item_menu_list = (in_item_menu_list + 3) * 2;
 
@@ -329,8 +340,7 @@ post_buffer_list(Widget w GCC_UNUSED,
 	    XmStringFree(xms);
 	    XtRemoveCallback(pm_buffer[nb_item_menu_list],
 			     XmNactivateCallback, list_proc_back, NULL);
-#endif
-#if ATHENA_WIDGETS
+#elif ATHENA_WIDGETS
 	    XtVaSetValues(pm_buffer[nb_item_menu_list],
 			  Nval(XtNlabel, string),
 			  NULL);
@@ -344,12 +354,11 @@ post_buffer_list(Widget w GCC_UNUSED,
 		      XmNactivateCallback,
 		      list_proc_back,
 		      NULL);
-#endif
-#if ATHENA_WIDGETS
+#elif ATHENA_WIDGETS
 	XtAddCallback(pm_buffer[nb_item_menu_list],
 		      XtNcallback,
 		      list_proc_back,
-		      (XtPointer) nb_item_menu_list);
+		      (XtPointer) (long) nb_item_menu_list);
 #endif
 	nb_item_menu_list++;
     }
@@ -375,30 +384,34 @@ gui_add_list_callback(void *pm)
 }
 
 /*
- * Hide/show menus, useful for avoiding thrashing while regenering menus
+ * Hide/show menus, useful for avoiding thrashing while regenerating menus
  */
 int
 gui_hide_menus(int f GCC_UNUSED, int n GCC_UNUSED)
 {
+    int rc = FALSE;
     Widget w = x_menu_widget();
 
     if (w != 0) {
 	XtUnmapWidget(w);
-	return TRUE;
+	rc = TRUE;
     }
-    return FALSE;
+    TRACE(("gui_hide_menus %d\n", rc));
+    return rc;
 }
 
 int
 gui_show_menus(int f GCC_UNUSED, int n GCC_UNUSED)
 {
+    int rc = FALSE;
     Widget w = x_menu_widget();
 
     if (w != 0) {
 	XtMapWidget(w);
-	return TRUE;
+	rc = TRUE;
     }
-    return FALSE;
+    TRACE(("gui_show_menus %d\n", rc));
+    return rc;
 }
 
 /*
@@ -407,16 +420,23 @@ gui_show_menus(int f GCC_UNUSED, int n GCC_UNUSED)
 int
 gui_remove_menus(int f GCC_UNUSED, int n GCC_UNUSED)
 {
+    int rc = TRUE;
+    int count = 0;
+
     gui_hide_menus(f, n);
     while (my_menus != 0) {
 	MY_MENUS *next = my_menus->next;
+
 	XtUnmanageChild(my_menus->parent);
 	XtDestroyWidget(my_menus->parent);
 	XtDestroyWidget(my_menus->child);
+
 	free(my_menus);
 	my_menus = next;
+	++count;
     }
-    return FALSE;
+    TRACE(("gui_remove_menus %d\n", count));
+    return rc;
 }
 
 /*
@@ -425,7 +445,9 @@ gui_remove_menus(int f GCC_UNUSED, int n GCC_UNUSED)
 int
 gui_create_menus(void)
 {
-    return do_menu(x_menu_widget());
+    int rc = do_menu(x_menu_widget());
+    TRACE(("gui_create_menus %d\n", rc));
+    return rc;
 }
 
 #endif /* DISP_X11 */
